@@ -19,8 +19,8 @@
 
 set -e
 
-if [ "$#" -ne 1 ]; then
-  echo "Usage: $0 <numba version>"
+if [ "$#" -ne 1 ] && [ "$#" -ne 2 ] && [ "$#" -ne 3 ]; then
+  echo "Usage: $0 <numba version> [numba-cuda version] [CUDA version]"
   exit 1
 fi
 
@@ -41,3 +41,26 @@ elif [ "${numba}" = "latest" ]; then
 else
   pip install "numba==${numba}"
 fi
+
+if [ "$#" -eq 1 ]; then
+  exit 0
+fi
+
+numba_cuda=$2
+
+DEFAULT_CUDA_VERSION="12"
+
+if [ "$#" -eq 3 ]; then
+  # Extract the CUDA major version only
+  cuda_version="${3%%.*}"
+else
+  cuda_version="${DEFAULT_CUDA_VERSION}"
+fi
+
+if [ "${numba_cuda}" = "master" ]; then
+  pip install "numba-cuda[cu${cuda_version}] @ https://github.com/NVIDIA/numba-cuda/archive/main.tar.gz"
+elif [ "${numba_cuda}" = "latest" ]; then
+  pip install "numba-cuda[cu${cuda_version}]"
+else
+  pip install "numba-cuda[cu${cuda_version}]==${numba_cuda}"
+fi
