@@ -267,11 +267,6 @@ class SinkNode : public ExecNode,
       return Status::Invalid(
           "`backpressure::pause_if_above` must be >= `backpressure::resume_if_below");
     }
-    if (sink_options.backpressure.resume_if_below < 0) {
-      return Status::Invalid(
-          "`backpressure::pause_if_above and backpressure::resume_if_below must be >= 0. "
-          " Set to 0 to disable backpressure.");
-    }
     return Status::OK();
   }
 
