@@ -22,16 +22,17 @@
 #include "arrow/acero/partition_util.h"
 #include "arrow/acero/schema_util.h"
 #include "arrow/acero/task_util.h"
-#include "arrow/compute/kernels/row_encoder_internal.h"
 #include "arrow/compute/key_map_internal.h"
 #include "arrow/compute/light_array_internal.h"
 #include "arrow/compute/row/encode_internal.h"
+#include "arrow/compute/row/row_encoder_internal.h"
 
 namespace arrow {
 
 using compute::ExecBatchBuilder;
 using compute::KeyColumnArray;
 using compute::KeyColumnMetadata;
+using compute::LightContext;
 using compute::ResizableArrayData;
 using compute::RowTableEncoder;
 using compute::RowTableImpl;
@@ -47,16 +48,6 @@ class RowArrayAccessor {
   //
   static int VarbinaryColumnId(const RowTableMetadata& row_metadata, int column_id);
 
-  // Calculate how many rows to skip from the tail of the
-  // sequence of selected rows, such that the total size of skipped rows is at
-  // least equal to the size specified by the caller. Skipping of the tail rows
-  // is used to allow for faster processing by the caller of remaining rows
-  // without checking buffer bounds (useful with SIMD or fixed size memory loads
-  // and stores).
-  //
-  static int NumRowsToSkip(const RowTableImpl& rows, int column_id, int num_rows,
-                           const uint32_t* row_ids, int num_tail_bytes_to_skip);
-
   // The supplied lambda will be called for each row in the given list of rows.
   // The arguments given to it will be:
   // - index of a row (within the set of selected rows),
@@ -68,7 +59,79 @@ class RowArrayAccessor {
   //
   template <class PROCESS_VALUE_FN>
   static void Visit(const RowTableImpl& rows, int column_id, int num_rows,
-                    const uint32_t* row_ids, PROCESS_VALUE_FN process_value_fn);
+                    const uint32_t* row_ids, PROCESS_VALUE_FN process_value_fn) {
+    bool is_fixed_length_column =
+        rows.metadata().column_metadatas[column_id].is_fixed_length;
+
+    // There are 4 cases, each requiring different steps:
+    // 1. Varying length column that is the first varying length column in a row
+    // 2. Varying length column that is not the first varying length column in a
+    // row
+    // 3. Fixed length column in a fixed length row
+    // 4. Fixed length column in a varying length row
+
+    if (!is_fixed_length_column) {
+      int varbinary_column_id = VarbinaryColumnId(rows.metadata(), column_id);
+      const uint8_t* row_ptr_base = rows.var_length_rows();
+      const RowTableImpl::offset_type* row_offsets = rows.offsets();
+      uint32_t field_offset_within_row, field_length;
+
+      if (varbinary_column_id == 0) {
+        // Case 1: This is the first varbinary column
+        //
+        for (int i = 0; i < num_rows; ++i) {
+          uint32_t row_id = row_ids[i];
+          const uint8_t* row_ptr = row_ptr_base + row_offsets[row_id];
+          rows.metadata().first_varbinary_offset_and_length(
+              row_ptr, &field_offset_within_row, &field_length);
+          process_value_fn(i, row_ptr + field_offset_within_row, field_length);
+        }
+      } else {
+        // Case 2: This is second or later varbinary column
+        //
+        for (int i = 0; i < num_rows; ++i) {
+          uint32_t row_id = row_ids[i];
+          const uint8_t* row_ptr = row_ptr_base + row_offsets[row_id];
+          rows.metadata().nth_varbinary_offset_and_length(
+              row_ptr, varbinary_column_id, &field_offset_within_row, &field_length);
+          process_value_fn(i, row_ptr + field_offset_within_row, field_length);
+        }
+      }
+    }
+
+    if (is_fixed_length_column) {
+      uint32_t field_offset_within_row = rows.metadata().encoded_field_offset(
+          rows.metadata().pos_after_encoding(column_id));
+      uint32_t field_length = rows.metadata().column_metadatas[column_id].fixed_length;
+      // Bit column is encoded as a single byte
+      //
+      if (field_length == 0) {
+        field_length = 1;
+      }
+
+      bool is_fixed_length_row = rows.metadata().is_fixed_length;
+      if (is_fixed_length_row) {
+        // Case 3: This is a fixed length column in a fixed length row
+        //
+        for (int i = 0; i < num_rows; ++i) {
+          uint32_t row_id = row_ids[i];
+          const uint8_t* row_ptr =
+              rows.fixed_length_rows(row_id) + field_offset_within_row;
+          process_value_fn(i, row_ptr, field_length);
+        }
+      } else {
+        // Case 4: This is a fixed length column in a varying length row
+        //
+        const uint8_t* row_ptr_base = rows.var_length_rows() + field_offset_within_row;
+        const RowTableImpl::offset_type* row_offsets = rows.offsets();
+        for (int i = 0; i < num_rows; ++i) {
+          uint32_t row_id = row_ids[i];
+          const uint8_t* row_ptr = row_ptr_base + row_offsets[row_id];
+          process_value_fn(i, row_ptr, field_length);
+        }
+      }
+    }
+  }
 
   // The supplied lambda will be called for each row in the given list of rows.
   // The arguments given to it will be:
@@ -77,9 +140,14 @@ class RowArrayAccessor {
   //
   template <class PROCESS_VALUE_FN>
   static void VisitNulls(const RowTableImpl& rows, int column_id, int num_rows,
-                         const uint32_t* row_ids, PROCESS_VALUE_FN process_value_fn);
+                         const uint32_t* row_ids, PROCESS_VALUE_FN process_value_fn) {
+    uint32_t pos_after_encoding = rows.metadata().pos_after_encoding(column_id);
+    for (int i = 0; i < num_rows; ++i) {
+      uint32_t row_id = row_ids[i];
+      process_value_fn(i, rows.is_null(row_id, pos_after_encoding) ? 0xff : 0);
+    }
+  }
 
- private:
 #if defined(ARROW_HAVE_RUNTIME_AVX2)
   // This is equivalent to Visit method, but processing 8 rows at a time in a
   // loop.
@@ -107,14 +175,16 @@ class RowArrayAccessor {
 // Read operations (row comparison, column decoding)
 // can be called by multiple threads concurrently.
 //
-struct RowArray {
-  RowArray() : is_initialized_(false) {}
+struct ARROW_ACERO_EXPORT RowArray {
+  RowArray() : is_initialized_(false), hardware_flags_(0) {}
 
-  Status InitIfNeeded(MemoryPool* pool, const ExecBatch& batch);
-  Status InitIfNeeded(MemoryPool* pool, const RowTableMetadata& row_metadata);
+  Status InitIfNeeded(MemoryPool* pool, int64_t hardware_flags, const ExecBatch& batch);
+  Status InitIfNeeded(MemoryPool* pool, int64_t hardware_flags,
+                      const RowTableMetadata& row_metadata);
 
-  Status AppendBatchSelection(MemoryPool* pool, const ExecBatch& batch, int begin_row_id,
-                              int end_row_id, int num_row_ids, const uint16_t* row_ids,
+  Status AppendBatchSelection(MemoryPool* pool, int64_t hardware_flags,
+                              const ExecBatch& batch, int begin_row_id, int end_row_id,
+                              int num_row_ids, const uint16_t* row_ids,
                               std::vector<KeyColumnArray>& temp_column_arrays);
 
   // This can only be called for a minibatch.
@@ -122,12 +192,10 @@ struct RowArray {
   void Compare(const ExecBatch& batch, int begin_row_id, int end_row_id, int num_selected,
                const uint16_t* batch_selection_maybe_null, const uint32_t* array_row_ids,
                uint32_t* out_num_not_equal, uint16_t* out_not_equal_selection,
-               int64_t hardware_flags, arrow::util::TempVectorStack* temp_stack,
+               arrow::util::TempVectorStack* temp_stack,
                std::vector<KeyColumnArray>& temp_column_arrays,
                uint8_t* out_match_bitvector_maybe_null = NULLPTR);
 
-  // TODO: add AVX2 version
-  //
   Status DecodeSelected(ResizableArrayData* target, int column_id, int num_rows_to_append,
                         const uint32_t* row_ids, MemoryPool* pool) const;
 
@@ -135,10 +203,43 @@ struct RowArray {
 
   int64_t num_rows() const { return is_initialized_ ? rows_.length() : 0; }
 
+  void EnsureHasAnyNullsComputed(const LightContext& ctx) {
+    std::ignore = rows_.has_any_nulls(&ctx);
+  }
+
+ private:
   bool is_initialized_;
+
+  int64_t hardware_flags_;
   RowTableEncoder encoder_;
   RowTableImpl rows_;
   RowTableImpl rows_temp_;
+
+ private:
+  void DecodeFixedLength(ResizableArrayData* output, int output_start_row, int column_id,
+                         uint32_t fixed_length, int num_rows_to_append,
+                         const uint32_t* row_ids) const;
+  void DecodeOffsets(ResizableArrayData* output, int output_start_row, int column_id,
+                     int num_rows_to_append, const uint32_t* row_ids) const;
+  void DecodeVarLength(ResizableArrayData* output, int output_start_row, int column_id,
+                       int num_rows_to_append, const uint32_t* row_ids) const;
+  void DecodeNulls(ResizableArrayData* output, int output_start_row, int column_id,
+                   int num_rows_to_append, const uint32_t* row_ids) const;
+
+#if defined(ARROW_HAVE_RUNTIME_AVX2)
+  int DecodeFixedLength_avx2(ResizableArrayData* output, int output_start_row,
+                             int column_id, uint32_t fixed_length, int num_rows_to_append,
+                             const uint32_t* row_ids) const;
+  int DecodeOffsets_avx2(ResizableArrayData* output, int output_start_row, int column_id,
+                         int num_rows_to_append, const uint32_t* row_ids) const;
+  int DecodeVarLength_avx2(ResizableArrayData* output, int output_start_row,
+                           int column_id, int num_rows_to_append,
+                           const uint32_t* row_ids) const;
+  int DecodeNulls_avx2(ResizableArrayData* output, int output_start_row, int column_id,
+                       int num_rows_to_append, const uint32_t* row_ids) const;
+#endif
+
+  friend class RowArrayMerge;
 };
 
 // Implements concatenating multiple row arrays into a single one, using
@@ -160,8 +261,8 @@ class RowArrayMerge {
   // caller can pass in nullptr to indicate that it is not needed.
   //
   static Status PrepareForMerge(RowArray* target, const std::vector<RowArray*>& sources,
-                                std::vector<int64_t>* first_target_row_id,
-                                MemoryPool* pool);
+                                std::vector<uint32_t>* first_target_row_id,
+                                MemoryPool* pool, int64_t hardware_flags);
 
   // Copy rows from source array to target array.
   // Both arrays must have the same row metadata.
@@ -185,8 +286,8 @@ class RowArrayMerge {
   // initializes boundary offsets for target row ranges for each source).
   //
   static void MergeSingle(RowArray* target, const RowArray& source,
-                          int64_t first_target_row_id,
-                          const int64_t* source_rows_permutation);
+                          uint32_t first_target_row_id,
+                          const uint32_t* source_rows_permutation);
 
  private:
   // Copy rows from source array to a region of the target array.
@@ -194,24 +295,24 @@ class RowArrayMerge {
   // Null information needs to be handled separately.
   //
   static void CopyFixedLength(RowTableImpl* target, const RowTableImpl& source,
-                              int64_t first_target_row_id,
-                              const int64_t* source_rows_permutation);
+                              uint32_t first_target_row_id,
+                              const uint32_t* source_rows_permutation);
 
   // Copy rows from source array to a region of the target array.
   // This implementation is for varying length rows.
   // Null information needs to be handled separately.
   //
   static void CopyVaryingLength(RowTableImpl* target, const RowTableImpl& source,
-                                int64_t first_target_row_id,
+                                uint32_t first_target_row_id,
                                 int64_t first_target_row_offset,
-                                const int64_t* source_rows_permutation);
+                                const uint32_t* source_rows_permutation);
 
   // Copy null information from rows from source array to a region of the target
   // array.
   //
   static void CopyNulls(RowTableImpl* target, const RowTableImpl& source,
-                        int64_t first_target_row_id,
-                        const int64_t* source_rows_permutation);
+                        uint32_t first_target_row_id,
+                        const uint32_t* source_rows_permutation);
 };
 
 // Implements merging of multiple SwissTables into a single one, using
@@ -279,8 +380,8 @@ class SwissTableMerge {
   // Max block id value greater or equal to the number of blocks guarantees that
   // the search will not be stopped.
   //
-  static inline bool InsertNewGroup(SwissTable* target, uint64_t group_id, uint32_t hash,
-                                    int64_t max_block_id);
+  static inline bool InsertNewGroup(SwissTable* target, uint32_t group_id, uint32_t hash,
+                                    uint32_t max_block_id);
 };
 
 struct SwissTableWithKeys {
@@ -422,19 +523,27 @@ class SwissTableForJoin {
 //
 class SwissTableForJoinBuild {
  public:
-  Status Init(SwissTableForJoin* target, int dop, int64_t num_rows,
+  Status Init(SwissTableForJoin* target, int dop, int64_t num_rows, int64_t num_batches,
               bool reject_duplicate_keys, bool no_payload,
               const std::vector<KeyColumnMetadata>& key_types,
               const std::vector<KeyColumnMetadata>& payload_types, MemoryPool* pool,
               int64_t hardware_flags);
 
-  // In the first phase of parallel hash table build, threads pick unprocessed
-  // exec batches, partition the rows based on hash, and update all of the
-  // partitions with information related to that batch of rows.
+  // In the first phase of parallel hash table build, each thread picks unprocessed exec
+  // batches, hashes the batches and preserve the hashes, then partition the rows based on
+  // hashes.
   //
-  Status PushNextBatch(int64_t thread_id, const ExecBatch& key_batch,
-                       const ExecBatch* payload_batch_maybe_null,
-                       arrow::util::TempVectorStack* temp_stack);
+  Status PartitionBatch(size_t thread_id, int64_t batch_id, const ExecBatch& key_batch,
+                        arrow::util::TempVectorStack* temp_stack);
+
+  // In the second phase of parallel hash table build, each thread picks the given
+  // partition of all batches, and updates that particular partition with information
+  // related to that batch of rows.
+  //
+  Status ProcessPartition(size_t thread_id, int64_t batch_id, int prtn_id,
+                          const ExecBatch& key_batch,
+                          const ExecBatch* payload_batch_maybe_null,
+                          arrow::util::TempVectorStack* temp_stack);
 
   // Allocate memory and initialize counters required for parallel merging of
   // hash table partitions.
@@ -442,7 +551,7 @@ class SwissTableForJoinBuild {
   //
   Status PreparePrtnMerge();
 
-  // Second phase of parallel hash table build.
+  // Third phase of parallel hash table build.
   // Each partition can be processed by a different thread.
   // Parallel step.
   //
@@ -463,9 +572,6 @@ class SwissTableForJoinBuild {
 
  private:
   void InitRowArray();
-  Status ProcessPartition(int64_t thread_id, const ExecBatch& key_batch,
-                          const ExecBatch* payload_batch_maybe_null,
-                          arrow::util::TempVectorStack* temp_stack, int prtn_id);
 
   SwissTableForJoin* target_;
   // DOP stands for Degree Of Parallelism - the maximum number of participating
@@ -503,6 +609,22 @@ class SwissTableForJoinBuild {
   MemoryPool* pool_;
   int64_t hardware_flags_;
 
+  // One per batch.
+  //
+  // Informations like hashes and partitions of each batch gathered in the partition phase
+  // and used in the build phase.
+  //
+  struct BatchState {
+    // Hashes for the batch, preserved in the partition phase to avoid recomputation in
+    // the build phase. One element per row in the batch.
+    std::vector<uint32_t> hashes;
+    // Accumulative number of rows in each partition for the batch. `num_prtns_` + 1
+    // elements.
+    std::vector<uint16_t> prtn_ranges;
+    // Row ids after partition sorting the batch. One element per row in the batch.
+    std::vector<uint16_t> prtn_row_ids;
+  };
+
   // One per partition.
   //
   struct PartitionState {
@@ -519,20 +641,16 @@ class SwissTableForJoinBuild {
   // batches.
   //
   struct ThreadState {
-    std::vector<uint32_t> batch_hashes;
-    std::vector<uint16_t> batch_prtn_ranges;
-    std::vector<uint16_t> batch_prtn_row_ids;
-    std::vector<int> temp_prtn_ids;
     std::vector<uint32_t> temp_group_ids;
     std::vector<KeyColumnArray> temp_column_arrays;
   };
 
+  std::vector<BatchState> batch_states_;
   std::vector<PartitionState> prtn_states_;
   std::vector<ThreadState> thread_states_;
-  PartitionLocks prtn_locks_;
 
-  std::vector<int64_t> partition_keys_first_row_id_;
-  std::vector<int64_t> partition_payloads_first_row_id_;
+  std::vector<uint32_t> partition_keys_first_row_id_;
+  std::vector<uint32_t> partition_payloads_first_row_id_;
 };
 
 class JoinResultMaterialize {
@@ -862,6 +980,8 @@ class JoinResidualFilter {
 
  private:
   Expression filter_;
+  bool is_trivial_ = false;
+  bool is_literal_true_ = false;
 
   QueryContext* ctx_;
   MemoryPool* pool_;
@@ -896,11 +1016,6 @@ class JoinProbeProcessor {
                      arrow::util::TempVectorStack* temp_stack,
                      std::vector<KeyColumnArray>* temp_column_arrays);
 
-  // Must be called by a single-thread having exclusive access to the instance
-  // of this class. The caller is responsible for ensuring that.
-  //
-  Status OnFinished();
-
  private:
   int num_key_columns_;
   JoinType join_type_;
