@@ -27,7 +27,7 @@
 #include "arrow/io/memory.h"
 #include "arrow/ipc/writer.h"
 #include "arrow/record_batch.h"
-#include "arrow/result_internal.h"
+#include "arrow/result.h"
 #include "arrow/testing/gtest_util.h"
 #include "arrow/testing/matchers.h"
 #include "arrow/type.h"
@@ -61,10 +61,12 @@ WriteOptions DefaultTestOptions(bool include_header = false,
                                 const std::string& null_string = "",
                                 QuotingStyle quoting_style = QuotingStyle::Needed,
                                 const std::string& eol = "\n", char delimiter = ',',
-                                int batch_size = 5) {
+                                int batch_size = 5,
+                                QuotingStyle quoting_header = QuotingStyle::Needed) {
   WriteOptions options;
   options.batch_size = batch_size;
   options.include_header = include_header;
+  options.quoting_header = quoting_header;
   options.null_string = null_string;
   options.eol = eol;
   options.quoting_style = quoting_style;
@@ -73,16 +75,17 @@ WriteOptions DefaultTestOptions(bool include_header = false,
 }
 
 std::string UtilGetExpectedWithEOL(const std::string& eol) {
-  return std::string("1,,-1,,,") + eol +        // line 1
-         R"(1,"abc""efg",2324,,,)" + eol +      // line 2
-         R"(,"abcd",5467,,,)" + eol +           // line 3
-         R"(,,,,,)" + eol +                     // line 4
-         R"(546,"",517,,,)" + eol +             // line 5
-         R"(124,"a""""b""",,,,)" + eol +        // line 6
-         R"(,,,1970-01-01,,)" + eol +           // line 7
-         R"(,,,,1970-01-02,)" + eol +           // line 8
-         R"(,,,,,2004-02-29 01:02:03)" + eol +  // line 9
-         R"(,"NA",,,,)" + eol;                  // line 10
+  return std::string("1,,-1,,,,,") + eol +        // line 1
+         R"(1,"abc""efg",2324,,,,,)" + eol +      // line 2
+         R"(,"abcd",5467,,,,,"efghi")" + eol +    // line 3
+         R"(,,,,,,,)" + eol +                     // line 4
+         R"(546,"",517,,,,,)" + eol +             // line 5
+         R"(124,"a""""b""",,,,,,)" + eol +        // line 6
+         R"(,,,1970-01-01,,,,"jklm")" + eol +     // line 7
+         R"(,,,,1970-01-02,,,)" + eol +           // line 8
+         R"(,,,,,2004-02-29 01:02:03,,)" + eol +  // line 9
+         R"(,,,,,,3600,)" + eol +                 // line 10
+         R"(,"NA",,,,,,)" + eol;                  // line 11
 }
 
 std::vector<WriterTestParams> GenerateTestCases() {
@@ -90,6 +93,17 @@ std::vector<WriterTestParams> GenerateTestCases() {
   auto dummy_schema = schema({field("a", uint8())});
   std::string dummy_batch_data = R"([{"a": null}])";
 
+  auto header_without_structural_charaters =
+      schema({field("a ", uint64()), field("b", int32())});
+  std::string expected_header_without_structural_charaters =
+      std::string(R"(a ,b)") + "\n";
+  auto expected_status_no_quotes_with_structural_in_header = [](const char* header) {
+    return Status::Invalid(
+        "CSV header may not contain structural characters if quoting "
+        "style is \"None\". See RFC4180. Invalid value: ",
+        header);
+  };
+
   // Schema to test various types.
   auto abc_schema = schema({
       field("a", uint64()),
@@ -98,19 +112,23 @@ std::vector<WriterTestParams> GenerateTestCases() {
       field("d", date32()),
       field("e", date64()),
       field("f", timestamp(TimeUnit::SECOND)),
+      field("g", duration(TimeUnit::SECOND)),
+      field("h", large_utf8()),
   });
   auto populated_batch = R"([{"a": 1, "c ": -1},
                              { "a": 1, "b\"": "abc\"efg", "c ": 2324},
-                             { "b\"": "abcd", "c ": 5467},
+                             { "b\"": "abcd", "c ": 5467, "h": "efghi"},
                              { },
                              { "a": 546, "b\"": "", "c ": 517 },
                              { "a": 124, "b\"": "a\"\"b\"" },
-                             { "d": 0 },
+                             { "d": 0, "h": "jklm" },
                              { "e": 86400000 },
                              { "f": 1078016523 },
+                             { "g": 3600 },
                              { "b\"": "NA" }])";
 
-  std::string expected_header = std::string(R"("a","b""","c ","d","e","f")") + "\n";
+  std::string expected_header =
+      std::string(R"("a","b""","c ","d","e","f","g","h")") + "\n";
 
   // Expected output without header when using default QuotingStyle::Needed.
   std::string expected_without_header = UtilGetExpectedWithEOL("\n");
@@ -119,39 +137,42 @@ std::vector<WriterTestParams> GenerateTestCases() {
 
   // Expected output without header when using QuotingStyle::AllValid.
   std::string expected_quoting_style_all_valid =
-      std::string(R"("1",,"-1",,,)") + "\n" +   // line 1
-      R"("1","abc""efg","2324",,,)" + "\n" +    // line 2
-      R"(,"abcd","5467",,,)" + "\n" +           // line 3
-      R"(,,,,,)" + "\n" +                       // line 4
-      R"("546","","517",,,)" + "\n" +           // line 5
-      R"("124","a""""b""",,,,)" + "\n" +        // line 6
-      R"(,,,"1970-01-01",,)" + "\n" +           // line 7
-      R"(,,,,"1970-01-02",)" + "\n" +           // line 8
-      R"(,,,,,"2004-02-29 01:02:03")" + "\n" +  // line 9
-      R"(,"NA",,,,)" + "\n";                    // line 10
+      std::string(R"("1",,"-1",,,,,)") + "\n" +   // line 1
+      R"("1","abc""efg","2324",,,,,)" + "\n" +    // line 2
+      R"(,"abcd","5467",,,,,"efghi")" + "\n" +    // line 3
+      R"(,,,,,,,)" + "\n" +                       // line 4
+      R"("546","","517",,,,,)" + "\n" +           // line 5
+      R"("124","a""""b""",,,,,,)" + "\n" +        // line 6
+      R"(,,,"1970-01-01",,,,"jklm")" + "\n" +     // line 7
+      R"(,,,,"1970-01-02",,,)" + "\n" +           // line 8
+      R"(,,,,,"2004-02-29 01:02:03",,)" + "\n" +  // line 9
+      R"(,,,,,,"3600",)" + "\n" +                 // line 10
+      R"(,"NA",,,,,,)" + "\n";                    // line 11
 
   // Batch when testing QuotingStyle::None. The values may not contain any quotes for this
   // style according to RFC4180.
   auto populated_batch_quoting_style_none = R"([{"a": 1, "c ": -1},
                              { "a": 1, "b\"": "abcefg", "c ": 2324},
-                             { "b\"": "abcd", "c ": 5467},
+                             { "b\"": "abcd", "c ": 5467, "h": "efghi"},
                              { },
                              { "a": 546, "b\"": "", "c ": 517 },
                              { "a": 124, "b\"": "ab" },
-                             { "d": 0 },
+                             { "d": 0, "h": "jklm" },
                              { "e": 86400000 },
-                             { "f": 1078016523 }])";
+                             { "f": 1078016523 },
+                             { "g": 3600 }])";
   // Expected output for QuotingStyle::None.
-  std::string expected_quoting_style_none = std::string("1,,-1,,,") + "\n" +  // line 1
-                                            R"(1,abcefg,2324,,,)" + "\n" +    // line 2
-                                            R"(,abcd,5467,,,)" + "\n" +       // line 3
-                                            R"(,,,,,)" + "\n" +               // line 4
-                                            R"(546,,517,,,)" + "\n" +         // line 5
-                                            R"(124,ab,,,,)" + "\n" +          // line 6
-                                            R"(,,,1970-01-01,,)" + "\n" +     // line 7
-                                            R"(,,,,1970-01-02,)" + "\n" +     // line 8
-                                            R"(,,,,,2004-02-29 01:02:03)" +
-                                            "\n";  // line 9
+  std::string expected_quoting_style_none = std::string("1,,-1,,,,,") + "\n" +   // line 1
+                                            R"(1,abcefg,2324,,,,,)" + "\n" +     // line 2
+                                            R"(,abcd,5467,,,,,efghi)" + "\n" +   // line 3
+                                            R"(,,,,,,,)" + "\n" +                // line 4
+                                            R"(546,,517,,,,,)" + "\n" +          // line 5
+                                            R"(124,ab,,,,,,)" + "\n" +           // line 6
+                                            R"(,,,1970-01-01,,,,jklm)" + "\n" +  // line 7
+                                            R"(,,,,1970-01-02,,,)" + "\n" +      // line 8
+                                            R"(,,,,,2004-02-29 01:02:03,,)" +
+                                            "\n" +                    // line 9
+                                            R"(,,,,,,3600,)" + "\n";  // line 10
 
   // Schema and data to test custom null value string.
   auto schema_custom_na = schema({field("g", uint64()), field("h", utf8())});
@@ -271,7 +292,20 @@ std::vector<WriterTestParams> GenerateTestCases() {
       {schema_custom_delimiter, batch_custom_delimiter,
        DefaultTestOptions(/*include_header=*/false, /*null_string=*/"",
                           QuotingStyle::Needed, /*eol=*/";", /*delimiter=*/';'),
-       /*expected_output*/ "", expected_status_illegal_delimiter(';')}};
+       /*expected_output*/ "", expected_status_illegal_delimiter(';')},
+      {header_without_structural_charaters, "[]",
+       DefaultTestOptions(/*include_header=*/true, /*null_string=*/"",
+                          QuotingStyle::Needed, /*eol=*/"\n",
+                          /*delimiter=*/',', /*batch_size=*/5,
+                          /*quoting_header=*/QuotingStyle::None),
+       expected_header_without_structural_charaters},
+      {abc_schema, "[]",
+       DefaultTestOptions(/*include_header=*/true, /*null_string=*/"",
+                          QuotingStyle::Needed, /*eol=*/"\n",
+                          /*delimiter=*/',', /*batch_size=*/5,
+                          /*quoting_header=*/QuotingStyle::None),
+       "", expected_status_no_quotes_with_structural_in_header("b\"")},
+  };
 }
 
 class TestWriteCSV : public ::testing::TestWithParam<WriterTestParams> {
@@ -279,19 +313,19 @@ class TestWriteCSV : public ::testing::TestWithParam<WriterTestParams> {
   template <typename Data>
   Result<std::string> ToCsvString(const Data& data, const WriteOptions& options) {
     std::shared_ptr<io::BufferOutputStream> out;
-    ASSIGN_OR_RAISE(out, io::BufferOutputStream::Create());
+    ARROW_ASSIGN_OR_RAISE(out, io::BufferOutputStream::Create());
 
     RETURN_NOT_OK(WriteCSV(data, options, out.get()));
-    ASSIGN_OR_RAISE(std::shared_ptr<Buffer> buffer, out->Finish());
+    ARROW_ASSIGN_OR_RAISE(std::shared_ptr<Buffer> buffer, out->Finish());
     return std::string(reinterpret_cast<const char*>(buffer->data()), buffer->size());
   }
 
   Result<std::string> ToCsvStringUsingWriter(const Table& data,
                                              const WriteOptions& options) {
     std::shared_ptr<io::BufferOutputStream> out;
-    ASSIGN_OR_RAISE(out, io::BufferOutputStream::Create());
+    ARROW_ASSIGN_OR_RAISE(out, io::BufferOutputStream::Create());
     // Write row-by-row
-    ASSIGN_OR_RAISE(auto writer, MakeCSVWriter(out, data.schema(), options));
+    ARROW_ASSIGN_OR_RAISE(auto writer, MakeCSVWriter(out, data.schema(), options));
     TableBatchReader reader(data);
     reader.set_chunksize(1);
     std::shared_ptr<RecordBatch> batch;
@@ -302,7 +336,7 @@ class TestWriteCSV : public ::testing::TestWithParam<WriterTestParams> {
     }
     RETURN_NOT_OK(writer->Close());
     EXPECT_EQ(data.num_rows(), writer->stats().num_record_batches);
-    ASSIGN_OR_RAISE(std::shared_ptr<Buffer> buffer, out->Finish());
+    ARROW_ASSIGN_OR_RAISE(std::shared_ptr<Buffer> buffer, out->Finish());
     return std::string(reinterpret_cast<const char*>(buffer->data()), buffer->size());
   }
 };
