@@ -1758,32 +1758,30 @@ Status FileTruncate(int fd, const int64_t size) {
 // Environment variables
 //
 
-Result<std::string> GetEnvVar(const char* name) {
+Result<std::string> GetEnvVar(std::string_view name) {
 #ifdef _WIN32
   // On Windows, getenv() reads an early copy of the process' environment
   // which doesn't get updated when SetEnvironmentVariable() is called.
   constexpr int32_t bufsize = 2000;
   char c_str[bufsize];
-  auto res = GetEnvironmentVariableA(name, c_str, bufsize);
+  auto res = GetEnvironmentVariableA(name.data(), c_str, bufsize);
   if (res >= bufsize) {
     return Status::CapacityError("environment variable value too long");
   } else if (res == 0) {
-    return Status::KeyError("environment variable undefined");
+    return Status::KeyError("environment variable '", name, "'undefined");
   }
   return std::string(c_str);
 #else
-  char* c_str = getenv(name);
+  char* c_str = getenv(name.data());
   if (c_str == nullptr) {
-    return Status::KeyError("environment variable undefined");
+    return Status::KeyError("environment variable '", name, "'undefined");
   }
   return std::string(c_str);
 #endif
 }
 
-Result<std::string> GetEnvVar(const std::string& name) { return GetEnvVar(name.c_str()); }
-
 #ifdef _WIN32
-Result<NativePathString> GetEnvVarNative(const std::string& name) {
+Result<NativePathString> GetEnvVarNative(std::string_view name) {
   NativePathString w_name;
   constexpr int32_t bufsize = 2000;
   wchar_t w_str[bufsize];
@@ -1793,33 +1791,28 @@ Result<NativePathString> GetEnvVarNative(const std::string& name) {
   if (res >= bufsize) {
     return Status::CapacityError("environment variable value too long");
   } else if (res == 0) {
-    return Status::KeyError("environment variable undefined");
+    return Status::KeyError("environment variable '", name, "'undefined");
   }
   return NativePathString(w_str);
 }
 
-Result<NativePathString> GetEnvVarNative(const char* name) {
-  return GetEnvVarNative(std::string(name));
-}
-
 #else
 
-Result<NativePathString> GetEnvVarNative(const std::string& name) {
+Result<NativePathString> GetEnvVarNative(std::string_view name) {
   return GetEnvVar(name);
 }
 
-Result<NativePathString> GetEnvVarNative(const char* name) { return GetEnvVar(name); }
 #endif
 
-Status SetEnvVar(const char* name, const char* value) {
+Status SetEnvVar(std::string_view name, std::string_view value) {
 #ifdef _WIN32
-  if (SetEnvironmentVariableA(name, value)) {
+  if (SetEnvironmentVariableA(name.data(), value.data())) {
     return Status::OK();
   } else {
     return Status::Invalid("failed setting environment variable");
   }
 #else
-  if (setenv(name, value, 1) == 0) {
+  if (setenv(name.data(), value.data(), 1) == 0) {
     return Status::OK();
   } else {
     return Status::Invalid("failed setting environment variable");
@@ -1827,19 +1820,15 @@ Status SetEnvVar(const char* name, const char* value) {
 #endif
 }
 
-Status SetEnvVar(const std::string& name, const std::string& value) {
-  return SetEnvVar(name.c_str(), value.c_str());
-}
-
-Status DelEnvVar(const char* name) {
+Status DelEnvVar(std::string_view name) {
 #ifdef _WIN32
-  if (SetEnvironmentVariableA(name, nullptr)) {
+  if (SetEnvironmentVariableA(name.data(), nullptr)) {
     return Status::OK();
   } else {
     return Status::Invalid("failed deleting environment variable");
   }
 #else
-  if (unsetenv(name) == 0) {
+  if (unsetenv(name.data()) == 0) {
     return Status::OK();
   } else {
     return Status::Invalid("failed deleting environment variable");
@@ -1847,8 +1836,6 @@ Status DelEnvVar(const char* name) {
 #endif
 }
 
-Status DelEnvVar(const std::string& name) { return DelEnvVar(name.c_str()); }
-
 //
 // Temporary directories
 //
