@@ -193,12 +193,22 @@ so_version() {
   expr ${major_version} \* 100 + ${minor_version}
 }
 
+gir_api_version() {
+  local version=$1
+  local major_version=$(echo ${version} | cut -d. -f1)
+  local minor_version=$(echo ${version} | cut -d. -f2)
+  echo "${major_version}.${minor_version}"
+}
+
 update_deb_package_names() {
   local version=$1
   local next_version=$2
   echo "Updating .deb package names for ${next_version}"
-  deb_lib_suffix=$(so_version ${version})
-  next_deb_lib_suffix=$(so_version ${next_version})
+
+  local substituted=false
+
+  local deb_lib_suffix=$(so_version ${version})
+  local next_deb_lib_suffix=$(so_version ${next_version})
   if [ "${deb_lib_suffix}" != "${next_deb_lib_suffix}" ]; then
     pushd ${ARROW_DIR}/dev/tasks/linux-packages/apache-arrow
     for target in debian*/lib*${deb_lib_suffix}.install; do
@@ -206,19 +216,34 @@ update_deb_package_names() {
         ${target} \
         $(echo ${target} | sed -e "s/${deb_lib_suffix}/${next_deb_lib_suffix}/")
     done
-    deb_lib_suffix_substitute_pattern="s/(lib(arrow|gandiva|parquet)[-a-z]*)${deb_lib_suffix}/\\1${next_deb_lib_suffix}/g"
+    local deb_lib_suffix_substitute_pattern="s/(lib(arrow|gandiva|parquet)[-a-z]*)${deb_lib_suffix}/\\1${next_deb_lib_suffix}/g"
     sed -i.bak -E -e "${deb_lib_suffix_substitute_pattern}" debian*/control*
     rm -f debian*/control*.bak
     git add debian*/control*
+    substituted=true
     popd
+  fi
 
-    pushd ${ARROW_DIR}/dev/release
-    sed -i.bak -E -e "${deb_lib_suffix_substitute_pattern}" rat_exclude_files.txt
-    rm -f rat_exclude_files.txt.bak
-    git add rat_exclude_files.txt
-    git commit -m "MINOR: [Release] Update .deb package names for ${next_version}"
+  local deb_gir_suffix=$(gir_api_version ${version})
+  local next_deb_gir_suffix=$(gir_api_version ${next_version})
+  if [ "${deb_gir_suffix}" != "${next_deb_gir_suffix}" ]; then
+    pushd ${ARROW_DIR}/dev/tasks/linux-packages/apache-arrow
+    for target in debian*/gir1.2-*-${deb_gir_suffix}.install; do
+      git mv \
+        ${target} \
+        $(echo ${target} | sed -e "s/${deb_gir_suffix}/${next_deb_gir_suffix}/")
+    done
+    local deb_gir_suffix_substitute_pattern="s/(gir1\\.2-(arrow|gandiva|parquet)[-a-z]*)${deb_gir_suffix}/\\1${next_deb_gir_suffix}/g"
+    sed -i.bak -E -e "${deb_gir_suffix_substitute_pattern}" debian*/control*
+    rm -f debian*/control*.bak
+    git add debian*/control*
+    substituted=true
     popd
   fi
+
+  if [ "${substituted}" = "true" ]; then
+    git commit -m "MINOR: [Release] Update .deb package names for ${next_version}"
+  fi
 }
 
 update_linux_packages() {
