@@ -21,14 +21,10 @@ set -exu
 
 if [ $# -lt 2 ]; then
   echo "Usage: $0 VERSION rc"
-  echo "       $0 VERSION staging-rc"
   echo "       $0 VERSION release"
-  echo "       $0 VERSION staging-release"
   echo "       $0 VERSION local"
   echo " e.g.: $0 0.13.0 rc                # Verify 0.13.0 RC"
-  echo " e.g.: $0 0.13.0 staging-rc        # Verify 0.13.0 RC on staging"
   echo " e.g.: $0 0.13.0 release           # Verify 0.13.0"
-  echo " e.g.: $0 0.13.0 staging-release   # Verify 0.13.0 on staging"
   echo " e.g.: $0 0.13.0-dev20210203 local # Verify 0.13.0-dev20210203 on local"
   exit 1
 fi
@@ -36,9 +32,11 @@ fi
 VERSION="$1"
 TYPE="$2"
 
-local_prefix="/arrow/dev/tasks/linux-packages"
+SOURCE_DIR="$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)"
+TOP_SOURCE_DIR="${SOURCE_DIR}/../.."
+local_prefix="${TOP_SOURCE_DIR}/dev/tasks/linux-packages"
 
-artifactory_base_url="https://apache.jfrog.io/artifactory/arrow"
+artifactory_base_url="https://packages.apache.org/artifactory/arrow"
 
 distribution=$(. /etc/os-release && echo "${ID}")
 distribution_version=$(. /etc/os-release && echo "${VERSION_ID}" | grep -o "^[0-9]*")
@@ -53,9 +51,10 @@ have_flight=yes
 have_gandiva=yes
 have_glib=yes
 have_parquet=yes
-have_python=yes
 have_ruby=yes
+have_vala=yes
 ruby_devel_packages=(ruby-devel)
+enablerepo_epel="--enablerepo=epel"
 install_command="dnf install -y --enablerepo=crb"
 uninstall_command="dnf remove -y"
 clean_command="dnf clean"
@@ -73,46 +72,20 @@ case "${distribution}-${distribution_version}" in
     ;;
   almalinux-*)
     distribution_prefix="almalinux"
-    ;;
-  amzn-2)
-    distribution_prefix="amazon-linux"
-    cmake_package=cmake3
-    cmake_command=cmake3
-    if [ "$(arch)" != "aarch64" ]; then
-      have_arrow_libs=yes
-    fi
-    have_flight=no
-    have_gandiva=no
-    have_python=no
-    have_ruby=no
-    install_command="yum install -y"
-    uninstall_command="yum remove -y"
-    clean_command="yum clean"
-    info_command="yum info"
-    amazon-linux-extras install epel -y
-    ;;
-  centos-7)
-    distribution_prefix="centos"
-    cmake_package=cmake3
-    cmake_command=cmake3
-    devtoolset=11
-    scl_package=centos-release-scl-rh
-    have_arrow_libs=yes
-    have_flight=no
-    have_gandiva=no
-    have_python=no
-    have_ruby=no
-    install_command="yum install -y"
-    uninstall_command="yum remove -y"
-    clean_command="yum clean"
-    info_command="yum info"
-    ;;
-  centos-8)
-    distribution_prefix="centos"
-    repository_version+="-stream"
     ruby_devel_packages+=(redhat-rpm-config)
-    install_command="dnf install -y --enablerepo=powertools"
-    info_command="dnf info --enablerepo=powertools"
+    ;;
+  amzn-*)
+    distribution_prefix="amazon-linux"
+    enablerepo_epel=""
+    # gcc-c++ installs kernel6.12-headers as a dependency
+    # automatically. But it conflicts with kernel-headers that is
+    # grpc-devel dependency. If we use "--allowerasing" when we
+    # install grpc-devel (via arrow-flight-devel), kernel6.12-headers
+    # is replaced with kernel-headers automatically. gcc-c++ works
+    # with kernel-headers too. So using kernel-headers is not a
+    # problem.
+    install_command="dnf install -y --allowerasing"
+    info_command="dnf info"
     ;;
   centos-*)
     distribution_prefix="centos"
@@ -120,7 +93,6 @@ case "${distribution}-${distribution_version}" in
     ;;
 esac
 if [ "$(arch)" = "aarch64" ]; then
-  have_flight=no
   have_gandiva=no
 fi
 
@@ -146,7 +118,6 @@ if [ "${TYPE}" = "local" ]; then
     amzn)
       package_version+=".${distribution}${distribution_version}"
       release_path+="/amazon-linux"
-      amazon-linux-extras install -y epel
       ;;
     centos)
       package_version+=".el${distribution_version}"
@@ -158,12 +129,10 @@ if [ "${TYPE}" = "local" ]; then
   ${install_command} "${release_path}"
 else
   package_version="${VERSION}"
-  case "${TYPE}" in
-    rc|staging-rc|staging-release)
-      suffix=${TYPE%-release}
-      distribution_prefix+="-${suffix}"
-      ;;
-  esac
+  if [ "${TYPE}" = "rc" ]; then
+    suffix=${TYPE%-release}
+    distribution_prefix+="-${suffix}"
+  fi
   ${install_command} \
     ${artifactory_base_url}/${distribution_prefix}/${repository_version}/apache-arrow-release-latest.rpm
 fi
@@ -171,34 +140,30 @@ fi
 if [ "${TYPE}" = "local" ]; then
   sed \
     -i"" \
-    -e "s,baseurl=https://apache\.jfrog\.io/artifactory/arrow/,baseurl=file://${local_prefix}/yum/repositories/,g" \
+    -e "s,baseurl=https://packages\.apache\.org/artifactory/arrow/,baseurl=file://${local_prefix}/yum/repositories/,g" \
     /etc/yum.repos.d/Apache-Arrow.repo
   keys="${local_prefix}/KEYS"
   if [ -f "${keys}" ]; then
     cp "${keys}" /etc/pki/rpm-gpg/RPM-GPG-KEY-Apache-Arrow
   fi
 else
-  case "${TYPE}" in
-    rc|staging-rc|staging-release)
-      suffix=${TYPE%-release}
-      sed \
-        -i"" \
-        -e "s,/almalinux/,/almalinux-${suffix}/,g" \
-        -e "s,/centos/,/centos-${suffix}/,g" \
-        -e "s,/amazon-linux/,/amazon-linux-${suffix}/,g" \
-        /etc/yum.repos.d/Apache-Arrow.repo
-      ;;
-  esac
+  if [ "${TYPE}" = "rc" ]; then
+    suffix=${TYPE%-release}
+    sed \
+      -i"" \
+      -e "s,/almalinux/,/almalinux-${suffix}/,g" \
+      -e "s,/centos/,/centos-${suffix}/,g" \
+      -e "s,/amazon-linux/,/amazon-linux-${suffix}/,g" \
+      /etc/yum.repos.d/Apache-Arrow.repo
+  fi
 fi
 
 echo "::endgroup::"
 
 
 echo "::group::Test Apache Arrow C++"
-${install_command} --enablerepo=epel arrow-devel-${package_version}
-if [ -n "${devtoolset}" ]; then
-  ${install_command} ${scl_package}
-fi
+mkdir -p build
+${install_command} ${enablerepo_epel} arrow-devel-${package_version}
 ${install_command} \
   ${cmake_package} \
   git \
@@ -214,34 +179,46 @@ else
     gcc-c++ \
     make
 fi
-mkdir -p build
-cp -a /arrow/cpp/examples/minimal_build build/
-pushd build/minimal_build
-${cmake_command} .
-make -j$(nproc)
-./arrow-example
-c++ -std=c++17 -o arrow-example example.cc $(pkg-config --cflags --libs arrow)
-./arrow-example
-popd
+# cmake version 3.31.6 -> 3.31.6
+cmake_version=$(${cmake_command} --version | head -n1 | sed -e 's/^cmake version //')
+# 3.31.6 -> 3.31
+cmake_version_major_minor=${cmake_version%.*}
+# 3.31 -> 3
+cmake_version_major=${cmake_version_major_minor%.*}
+# 3.31 -> 31
+cmake_version_minor=${cmake_version_major_minor#*.}
+if [ "${cmake_version_major}" -gt "3" ] || \
+   [ "${cmake_version_major}" -eq "3" -a "${cmake_version_minor}" -ge "25" ]; then
+  cp -a "${TOP_SOURCE_DIR}/cpp/examples/minimal_build" build/
+  pushd build/minimal_build
+  ${cmake_command} .
+  make -j$(nproc)
+  ./arrow-example
+  c++ -o arrow-example example.cc $(pkg-config --cflags --libs arrow) -std=c++2a
+  ./arrow-example
+  popd
+fi
 echo "::endgroup::"
 
 if [ "${have_glib}" = "yes" ]; then
   echo "::group::Test Apache Arrow GLib"
   export G_DEBUG=fatal-warnings
 
-  ${install_command} --enablerepo=epel arrow-glib-devel-${package_version}
-  ${install_command} --enablerepo=epel arrow-glib-doc-${package_version}
+  ${install_command} ${enablerepo_epel} arrow-glib-devel-${package_version}
+  ${install_command} ${enablerepo_epel} arrow-glib-doc-${package_version}
 
-  ${install_command} vala
-  cp -a /arrow/c_glib/example/vala build/
-  pushd build/vala
-  valac --pkg arrow-glib --pkg posix build.vala
-  ./build
-  popd
+  if [ "${have_vala}" = "yes" ]; then
+    ${install_command} vala
+    cp -a "${TOP_SOURCE_DIR}/c_glib/example/vala" build/
+    pushd build/vala
+    valac --pkg arrow-glib --pkg posix build.vala
+    ./build
+    popd
+  fi
 
   if [ "${have_ruby}" = "yes" ]; then
     ${install_command} "${ruby_devel_packages[@]}"
-    gem install gobject-introspection
+    MAKEFLAGS="-j$(nproc)" gem install gobject-introspection
     ruby -r gi -e "p GI.load('Arrow')"
   fi
   echo "::endgroup::"
@@ -249,44 +226,32 @@ fi
 
 if [ "${have_flight}" = "yes" ]; then
   echo "::group::Test Apache Arrow Flight"
-  ${install_command} --enablerepo=epel arrow-flight-glib-devel-${package_version}
-  ${install_command} --enablerepo=epel arrow-flight-glib-doc-${package_version}
+  ${install_command} ${enablerepo_epel} arrow-flight-glib-devel-${package_version}
+  ${install_command} ${enablerepo_epel} arrow-flight-glib-doc-${package_version}
   if [ "${have_ruby}" = "yes" ]; then
     ruby -r gi -e "p GI.load('ArrowFlight')"
   fi
   echo "::endgroup::"
 
   echo "::group::Test Apache Arrow Flight SQL"
-  ${install_command} --enablerepo=epel arrow-flight-sql-glib-devel-${package_version}
-  ${install_command} --enablerepo=epel arrow-flight-sql-glib-doc-${package_version}
+  ${install_command} ${enablerepo_epel} arrow-flight-sql-glib-devel-${package_version}
+  ${install_command} ${enablerepo_epel} arrow-flight-sql-glib-doc-${package_version}
   if [ "${have_ruby}" = "yes" ]; then
     ruby -r gi -e "p GI.load('ArrowFlightSQL')"
   fi
   echo "::endgroup::"
 fi
 
-echo "::group::Test Plasma"
-if [ "${have_glib}" = "yes" ]; then
-  ${install_command} --enablerepo=epel plasma-glib-devel-${package_version}
-  ${install_command} --enablerepo=epel plasma-glib-doc-${package_version}
-  if [ "${have_ruby}" = "yes" ]; then
-    ruby -r gi -e "p GI.load('Plasma')"
-  fi
-else
-  ${install_command} --enablerepo=epel plasma-devel-${package_version}
-fi
-echo "::endgroup::"
-
 if [ "${have_gandiva}" = "yes" ]; then
   echo "::group::Test Gandiva"
   if [ "${have_glib}" = "yes" ]; then
-    ${install_command} --enablerepo=epel gandiva-glib-devel-${package_version}
-    ${install_command} --enablerepo=epel gandiva-glib-doc-${package_version}
+    ${install_command} ${enablerepo_epel} gandiva-glib-devel-${package_version}
+    ${install_command} ${enablerepo_epel} gandiva-glib-doc-${package_version}
     if [ "${have_ruby}" = "yes" ]; then
       ruby -r gi -e "p GI.load('Gandiva')"
     fi
   else
-    ${install_command} --enablerepo=epel gandiva-devel-${package_version}
+    ${install_command} ${enablerepo_epel} gandiva-devel-${package_version}
   fi
   echo "::endgroup::"
 fi
@@ -294,13 +259,13 @@ fi
 if [ "${have_parquet}" = "yes" ]; then
   echo "::group::Test Apache Parquet"
   if [ "${have_glib}" = "yes" ]; then
-    ${install_command} --enablerepo=epel parquet-glib-devel-${package_version}
-    ${install_command} --enablerepo=epel parquet-glib-doc-${package_version}
+    ${install_command} ${enablerepo_epel} parquet-glib-devel-${package_version}
+    ${install_command} ${enablerepo_epel} parquet-glib-doc-${package_version}
     if [ "${have_ruby}" = "yes" ]; then
       ruby -r gi -e "p GI.load('Parquet')"
     fi
   else
-    ${install_command} --enablerepo=epel parquet-devel-${package_version}
+    ${install_command} ${enablerepo_epel} parquet-devel-${package_version}
   fi
   echo "::endgroup::"
 fi
@@ -308,15 +273,16 @@ fi
 echo "::group::Test coexistence with old library"
 ${uninstall_command} apache-arrow-release
 if ${install_command} \
-     https://apache.jfrog.io/artifactory/arrow/${distribution_prefix}/${repository_version}/apache-arrow-release-latest.rpm; then
+     ${artifactory_base_url}/${distribution_prefix}/${repository_version}/apache-arrow-release-latest.rpm; then
   ${clean_command} all
   if [ "${have_arrow_libs}" = "yes" ]; then
-    ${install_command} arrow-libs
+    ${install_command} ${enablerepo_epel} arrow-libs
   else
     major_version=$(echo ${VERSION} | grep -E -o '^[0-9]+')
     previous_major_version="$((${major_version} - 1))"
-    if ${info_command} arrow${previous_major_version}-libs; then
-      ${install_command} arrow${previous_major_version}-libs
+    previous_so_version="${previous_major_version}00"
+    if ${info_command} ${enablerepo_epel} arrow${previous_so_version}-libs; then
+      ${install_command} ${enablerepo_epel} arrow${previous_so_version}-libs
     fi
   fi
 fi
