@@ -15,13 +15,12 @@
 # specific language governing permissions and limitations
 # under the License.
 
-
 test_that("Alternate type names are supported", {
   expect_equal(
     schema(b = double(), c = bool(), d = string(), e = float(), f = halffloat()),
     schema(b = float64(), c = boolean(), d = utf8(), e = float32(), f = float16())
   )
-  expect_equal(names(schema(b = double(), c = bool(), d = string())), c("b", "c", "d"))
+  expect_named(schema(b = double(), c = bool(), d = string()), c("b", "c", "d"))
 })
 
 test_that("Schema print method", {
@@ -38,11 +37,43 @@ test_that("Schema print method", {
   )
 })
 
+test_that("Schema$code()", {
+  expect_code_roundtrip(
+    schema(a = int32(), b = struct(c = double(), d = utf8()), e = list_of(binary()))
+  )
+
+  skip_if(packageVersion("rlang") < "1")
+  expect_error(
+    eval(schema(x = int32(), y = DayTimeInterval__initialize())$code()),
+    "Unsupported type"
+  )
+})
+
+test_that("Schema with non-nullable fields", {
+  expect_output(
+    print(
+      schema(
+        field("b", double()),
+        field("c", bool(), nullable = FALSE),
+        field("d", string())
+      )
+    ),
+    paste(
+      "Schema",
+      "b: double",
+      "c: bool not null",
+      "d: string",
+      sep = "\n"
+    ),
+    fixed = TRUE
+  )
+})
+
 test_that("Schema $GetFieldByName", {
   schm <- schema(b = double(), c = string())
   expect_equal(schm$GetFieldByName("b"), field("b", double()))
   expect_null(schm$GetFieldByName("f"))
-  # TODO: schema(b = double(), b = string())$GetFieldByName("b")
+  # TODO: schema(b = double(), b = string())$GetFieldByName("b") # nolint
   # also returns NULL and probably should error bc duplicated names
 })
 
@@ -108,6 +139,22 @@ test_that("Schema modification", {
   expect_error(schm[[c(2, 4)]] <- int32(), "length(i) not equal to 1", fixed = TRUE)
 })
 
+test_that("Metadata can be reassigned as a whole", {
+  schm <- schema(b = double(), c = string(), d = int8())
+
+  # Check named character vector
+  schm$metadata <- c("foo" = "bar")
+  expect_identical(schm$metadata, list(foo = "bar"))
+
+  # Check list()
+  schm$metadata <- list("foo" = "bar")
+  expect_identical(schm$metadata, list(foo = "bar"))
+
+  # Check NULL for removal
+  schm$metadata <- NULL
+  expect_identical(schm$metadata, set_names(list(), character()))
+})
+
 test_that("Metadata is preserved when modifying Schema", {
   schm <- schema(b = double(), c = string(), d = int8())
   schm$metadata$foo <- "bar"
@@ -153,14 +200,14 @@ test_that("Input validation when creating a table with a schema", {
 
 test_that("Schema$Equals", {
   a <- schema(b = double(), c = bool())
-  b <- a$WithMetadata(list(some="metadata"))
+  b <- a$WithMetadata(list(some = "metadata"))
 
   # different metadata
   expect_failure(expect_equal(a, b))
   expect_false(a$Equals(b, check_metadata = TRUE))
 
   # Metadata not checked
-  expect_equivalent(a, b)
+  expect_equal(a, b, ignore_attr = TRUE)
 
   # Non-schema object
   expect_false(a$Equals(42))
@@ -202,3 +249,83 @@ test_that("Schema to C-interface", {
   # must clean up the pointer or we leak
   delete_arrow_schema(ptr)
 })
+
+test_that("Schemas from lists", {
+  name_list_schema <- schema(list(b = double(), c = string(), d = int8()))
+
+  field_list_schema <- schema(
+    list(
+      field("b", double()),
+      field("c", bool()),
+      field("d", string())
+    )
+  )
+
+  expect_equal(name_list_schema, schema(b = double(), c = string(), d = int8()))
+  expect_equal(field_list_schema, schema(b = double(), c = bool(), d = string()))
+})
+
+test_that("as_schema() works for Schema objects", {
+  schema <- schema(col1 = int32())
+  expect_identical(as_schema(schema), schema)
+})
+
+test_that("as_schema() works for StructType objects", {
+  struct_type <- struct(col1 = int32())
+  expect_equal(as_schema(struct_type), schema(col1 = int32()))
+})
+
+test_that("schema name assignment", {
+  schm <- schema(x = int8(), y = string(), z = double())
+  expect_named(schm, c("x", "y", "z"))
+  names(schm) <- c("a", "b", "c")
+  expect_named(schm, c("a", "b", "c"))
+  expect_error(names(schm) <- "f", regexp = "Replacement names must contain same number of items as current names")
+  expect_error(names(schm) <- NULL, regexp = "Replacement names must be character vector, not NULL")
+
+  # Test that R metadata is updated appropriately
+  df <- data.frame(x = 1:3, y = c("a", "b", "c"))
+  schm2 <- arrow_table(df)$schema
+  names(schm2) <- c("col1", "col2")
+  expect_named(schm2, c("col1", "col2"))
+  expect_named(schm2$r_metadata$columns, c("col1", "col2"))
+})
+
+test_that("schema extraction", {
+  skip_if_not_available("dataset")
+
+  tbl <- arrow_table(example_data)
+  expect_equal(schema(example_data), tbl$schema)
+  expect_equal(schema(tbl), tbl$schema)
+
+  expect_equal(
+    schema(data.frame(a = 1, a = "x", check.names = FALSE, stringsAsFactors = FALSE)),
+    schema(a = double(), a = string())
+  )
+
+  expect_equal(schema(data.frame()), schema())
+
+  ds <- InMemoryDataset$create(example_data)
+  expect_equal(schema(ds), ds$schema)
+
+  rdr <- RecordBatchReader$create(record_batch(example_data))
+  expect_equal(schema(rdr), rdr$schema)
+
+  adq <- as_adq(example_data)
+  expect_equal(schema(adq), adq$.data$schema)
+})
+
+test_that("schema print truncation", {
+  tbl <- arrow_table(example_data)
+  out <- print_schema_fields(schema(tbl), truncate = TRUE, max_fields = 1)
+  expect_output(
+    cat(out),
+    "int: int32\n...\n6 more columns\nUse `schema()` to see entire schema",
+    fixed = TRUE
+  )
+
+  expect_error(
+    print_schema_fields(schema(tbl), truncate = TRUE, max_fields = 0),
+    regexp = "max_fields not greater than 0"
+  )
+})
