@@ -15,37 +15,86 @@
 # specific language governing permissions and limitations
 # under the License.
 
-args <- commandArgs(TRUE)
-VERSION <- args[1]
-dst_dir <- paste0("libarrow/arrow-", VERSION)
-
-# TESTING is set in test-nixlibs.R; it won't be set when called from configure
-test_mode <- exists("TESTING")
-
-# Prevent error with binary selection during testing.
-if (test_mode && is.na(VERSION)) {
-  VERSION <- "8.0.0.9000"
-}
-
-dev_version <- package_version(VERSION)[1, 4]
-
-# Small dev versions are added for R-only changes during CRAN submission.
-if (is.na(dev_version) || dev_version < 100) {
-  VERSION <- package_version(VERSION)[1, 1:3]
-  arrow_repo <- paste0(getOption("arrow.repo", sprintf("https://apache.jfrog.io/artifactory/arrow/r/%s", VERSION)), "/libarrow/")
-} else {
-  arrow_repo <- paste0(getOption("arrow.dev_repo", "https://nightlies.apache.org/arrow/r"), "/libarrow/")
-}
-
-options(.arrow.cleanup = character()) # To collect dirs to rm on exit
-on.exit(unlink(getOption(".arrow.cleanup")))
-
+#### Functions #### check end of file for main logic
 env_is <- function(var, value) identical(tolower(Sys.getenv(var)), value)
 
+# Log messages in the style of the configure script
+lg <- function(..., .indent = "***") {
+  cat(.indent, " ", sprintf(...), "\n", sep = "")
+}
+
+cleanup <- function(path) {
+  options(.arrow.cleanup = c(getOption(".arrow.cleanup"), path))
+}
+
+# Exit the script after logging with .status=1 instead of throwing an error
+exit <- function(..., .status = 1) {
+  lg(...)
+  q(save = "no", status = .status)
+}
+
+
+# checks the nightly repo for the latest nightly version X.Y.Z.100<dev>
+find_latest_nightly <- function(
+  description_version,
+  list_uri = "https://nightlies.apache.org/arrow/r/src/contrib/PACKAGES",
+  hush = quietly
+) {
+  if (!startsWith(arrow_repo, "https://nightlies.apache.org/arrow/r")) {
+    lg("Detected non standard dev repo: %s, not checking latest nightly version.", arrow_repo)
+    return(description_version)
+  }
+
+  res <- try(
+    {
+      # Binaries are only uploaded if all jobs pass so can just look at the source versions.
+      urls <- readLines(list_uri)
+      versions <- grep("Version:\\s*.*?", urls, value = TRUE)
+      versions <- sort(package_version(sub("Version:\\s*", "\\1", versions)))
+      major_versions <- versions$major
+
+      description_version_major <- as.integer(description_version[1, 1])
+      matching_major <- major_versions == description_version_major
+      if (!any(matching_major)) {
+        lg(
+          "No nightly binaries were found for version %s: falling back to libarrow build from source",
+          description_version
+        )
+
+        return(description_version)
+      }
+
+      versions <- versions[matching_major]
+      max(versions)
+    },
+    silent = hush
+  )
+
+  if (inherits(res, "try-error")) {
+    lg("Failed to find latest nightly for %s", description_version)
+    latest <- description_version
+  } else {
+    lg("Latest available nightly for %s: %s", description_version, res)
+    latest <- res
+  }
+  latest
+}
+
 try_download <- function(from_url, to_file, hush = quietly) {
+  if (!download_ok) {
+    # Don't even try
+    return(FALSE)
+  }
+  # We download some fairly large files, so ensure the timeout is set appropriately.
+  # This assumes a static library size of 100 MB (our current biggest is 78 MB) and
+  # a download speed of 0.2 MB/s (slow). This is to anticipate slower user connections
+  # or load on artifactory servers.
+  opts <- options(timeout = max(600, getOption("timeout")))
+  on.exit(options(opts))
+
   status <- try(
     suppressWarnings(
-      download.file(from_url, to_file, quiet = hush)
+      download.file(from_url, to_file, quiet = hush, mode = "wb")
     ),
     silent = hush
   )
@@ -53,52 +102,88 @@ try_download <- function(from_url, to_file, hush = quietly) {
   !inherits(status, "try-error") && status == 0
 }
 
-not_cran <- env_is("NOT_CRAN", "true")
-if (not_cran) {
-  # Set more eager defaults
-  if (env_is("LIBARROW_BINARY", "")) {
-    Sys.setenv(LIBARROW_BINARY = "true")
-  }
-  if (env_is("LIBARROW_MINIMAL", "")) {
-    Sys.setenv(LIBARROW_MINIMAL = "false")
-  }
-}
+validate_checksum <- function(binary_url, libfile, hush = quietly) {
+  # Explicitly setting the env var to "false" will skip checksum validation
+  # e.g. in case the included checksums are stale.
+  skip_checksum <- env_is("ARROW_R_ENFORCE_CHECKSUM", "false")
+  enforce_checksum <- env_is("ARROW_R_ENFORCE_CHECKSUM", "true")
+  checksum_path <- Sys.getenv("ARROW_R_CHECKSUM_PATH", "tools/checksums")
+  # validate binary checksum for CRAN release only
+  if (!skip_checksum && dir.exists(checksum_path) && is_release || enforce_checksum) {
+    # Munge the path to the correct sha file which we include during the
+    # release process
+    if (VERSION_22_OR_LATER) {
+      checksum_file <- sub(".+/(.+\\.zip)", "\\1\\.sha512", binary_url)
+    } else {
+      checksum_file <- sub(".+/bin/(.+\\.zip)", "\\1\\.sha512", binary_url)
+    }
+    checksum_file <- file.path(checksum_path, checksum_file)
 
-# For local debugging, set ARROW_R_DEV=TRUE to make this script print more
-quietly <- !env_is("ARROW_R_DEV", "true")
+    # Try `shasum`, and if that doesn't work, fall back to `sha512sum` if not found
+    # system2 doesn't generate an R error, so we can't use a tryCatch to
+    # move from shasum to sha512sum.
+    # The warnings from system2 if it fails pop up later in the log and thus are
+    # more confusing than they are helpful (so we suppress them)
+    checksum_ok <- suppressWarnings(system2(
+      "shasum",
+      args = c("--status", "-a", "512", "-c", checksum_file),
+      stdout = ifelse(quietly, FALSE, ""),
+      stderr = ifelse(quietly, FALSE, "")
+    )) ==
+      0
 
-# The default will build from source as a fallback if a binary is not found or shouldn't be used
-# Set LIBARROW_BUILD=FALSE to ensure that we use a previously built libarrow
-# and don't fall back to a full source build
-build_ok <- !env_is("LIBARROW_BUILD", "false")
+    if (!checksum_ok) {
+      checksum_ok <- suppressWarnings(system2(
+        "sha512sum",
+        args = c("--status", "-c", checksum_file),
+        stdout = ifelse(quietly, FALSE, ""),
+        stderr = ifelse(quietly, FALSE, "")
+      )) ==
+        0
+    }
 
-# Check if we're authorized to download (not asked an offline build).
-# (Note that cmake will still be downloaded if necessary
-#  https://arrow.apache.org/docs/developers/cpp/building.html#offline-builds)
-download_ok <- !test_mode && !env_is("TEST_OFFLINE_BUILD", "true")
-
-# This "tools/thirdparty_dependencies" path, within the tar file, might exist if
-# create_package_with_all_dependencies() was run, or if someone has created it
-# manually before running make build.
-# If you change this path, you also need to edit
-# `create_package_with_all_dependencies()` in install-arrow.R
-thirdparty_dependency_dir <- Sys.getenv("ARROW_THIRDPARTY_DEPENDENCY_DIR", "tools/thirdparty_dependencies")
-
-
-download_binary <- function(lib) {
-  libfile <- tempfile()
-  binary_url <- paste0(arrow_repo, "bin/", lib, "/arrow-", VERSION, ".zip")
-  if (try_download(binary_url, libfile)) {
-    if (!quietly) {
-      cat(sprintf("*** Successfully retrieved C++ binaries (%s)\n", lib))
+    if (checksum_ok) {
+      lg("Checksum validated successfully for libarrow")
+    } else {
+      lg("Checksum validation failed for libarrow")
+      unlink(libfile)
     }
   } else {
+    checksum_ok <- TRUE
+  }
+
+  # Return whether the checksum was successful
+  checksum_ok
+}
+
+download_binary <- function(lib) {
+  if (VERSION_22_OR_LATER) {
+    libfile <- paste0("r-libarrow-", lib, "-", VERSION, ".zip")
+    binary_url <- paste0(arrow_repo, libfile)
+  } else {
+    libfile <- paste0("arrow-", VERSION, ".zip")
+    binary_url <- paste0(arrow_repo, "bin/", lib, "/arrow-", VERSION, ".zip")
+  }
+  if (try_download(binary_url, libfile) && validate_checksum(binary_url, libfile)) {
+    lg("Successfully retrieved libarrow (%s)", lib)
+  } else {
+    # If the download or checksum fail, we will set libfile to NULL this will
+    # normally result in a source build after this.
+    # TODO: should we condense these together and only call them when verbose?
+    lg(
+      "Unable to retrieve libarrow for version %s (%s)",
+      VERSION,
+      lib
+    )
     if (!quietly) {
-      cat(sprintf("*** Downloading libarrow binary failed for version %s (%s)\n    at %s\n",
-                  VERSION, lib, binary_url))
+      lg(
+        "Attempted to download the libarrow binary from: %s",
+        binary_url
+      )
     }
     libfile <- NULL
   }
+
   libfile
 }
 
@@ -107,16 +192,24 @@ download_binary <- function(lib) {
 # of action based on the current system. Other values you can set it to:
 # * "FALSE" (not case-sensitive), to skip this option altogether
 # * "TRUE" (not case-sensitive), to try to discover your current OS, or
-# * Some other string: a "linux-openssl-${OPENSSL_VERSION}" that corresponds to
-#   a binary that is available, to override what this function may discover by
-#   default.
+# * Some other string: a binary identifier that corresponds to a binary that is
+#   available, to override what this function may discover by default.
 #   Possible values are:
-#    * "linux-openssl-1.0" (OpenSSL 1.0)
-#    * "linux-openssl-1.1" (OpenSSL 1.1)
-#    * "linux-openssl-3.0" (OpenSSL 3.0)
+#    * "linux-x86_64"
+#    * "darwin-arm64"
+#    * "darwin-x86_64"
+#    * "windows-x86_64"
 #   These string values, along with `NULL`, are the potential return values of
 #   this function.
 identify_binary <- function(lib = Sys.getenv("LIBARROW_BINARY"), info = distro()) {
+  if (on_windows) {
+    if (VERSION_22_OR_LATER) {
+      return("windows-x86_64")
+    } else {
+      return("windows")
+    }
+  }
+
   lib <- tolower(lib)
   if (identical(lib, "")) {
     # Not specified. Check the allowlist.
@@ -125,112 +218,147 @@ identify_binary <- function(lib = Sys.getenv("LIBARROW_BINARY"), info = distro()
 
   if (identical(lib, "false")) {
     # Do not download a binary
-    NULL
+    lib <- NULL
   } else if (!identical(lib, "true")) {
     # Env var provided an os-version to use, to override our logic.
     # We don't validate that this exists. If it doesn't, the download will fail
     # and the build will fall back to building from source
-    lib
+    if (grepl("openssl-1", lib)) {
+      stop(
+        "OpenSSL 1.x binaries are no longer provided. Use LIBARROW_BINARY='",
+        sub("-openssl-1.*$", "", lib),
+        "'"
+      )
+    }
+    if (grepl("openssl-3", lib)) {
+      lib <- sub("-openssl-3.*$", "", lib)
+      lg("OpenSSL suffix deprecated in LIBARROW_BINARY, using '%s'", lib)
+    }
   } else {
     # See if we can find a suitable binary
-    select_binary()
+    lib <- select_binary()
   }
+  lib
 }
 
-check_allowlist <- function(os, allowed = "https://raw.githubusercontent.com/apache/arrow/main/r/tools/nixlibs-allowlist.txt") {
+check_allowlist <- function(
+  os,
+  allowed = "https://raw.githubusercontent.com/apache/arrow/main/r/tools/nixlibs-allowlist.txt"
+) {
   allowlist <- tryCatch(
     # Try a remote allowlist so that we can add/remove without a release
     suppressWarnings(readLines(allowed)),
-    # Fallback to default: allowed only on Ubuntu and CentOS/RHEL
-    error = function(e) c("ubuntu", "centos", "redhat", "rhel")
+    # Fallback to default allow list shipped with the package
+    error = function(e) readLines("tools/nixlibs-allowlist.txt")
   )
   # allowlist should contain valid regular expressions (plain strings ok too)
   any(grepl(paste(allowlist, collapse = "|"), os))
 }
 
-select_binary <- function(os = tolower(Sys.info()[["sysname"]]),
-                          arch = tolower(Sys.info()[["machine"]]),
-                          test_program = test_for_curl_and_openssl) {
-  if (identical(os, "linux") && identical(arch, "x86_64")) {
-    # We only host x86 linux binaries today
-    tryCatch(
+select_binary <- function(
+  os = tolower(Sys.info()[["sysname"]]),
+  arch = tolower(Sys.info()[["machine"]]),
+  test_program = test_for_curl_and_openssl
+) {
+  if (identical(os, "darwin") || (identical(os, "linux") && identical(arch, "x86_64"))) {
+    # We only host x86 linux binaries and x86 & arm64 macos today
+    binary <- tryCatch(
       # Somehow the test program system2 call errors on the sanitizer builds
       # so globally handle the possibility that this could fail
       {
         errs <- compile_test_program(test_program)
-        determine_binary_from_stderr(errs)
+        if (has_binary_sysreqs(errs)) {
+          paste0(os, "-", arch)
+        } else {
+          NULL
+        }
       },
       error = function(e) {
-        cat("*** Unable to find libcurl and openssl\n")
+        lg("Unable to find libcurl and openssl")
         NULL
       }
     )
   } else {
     # No binary available for arch
-    cat(sprintf("*** Building on %s %s\n", os, arch))
-    NULL
+    lg("Building on %s %s", os, arch)
+    binary <- NULL
   }
+  binary
 }
 
 # This tests that curl and OpenSSL are present (bc we can include their headers)
 # and it checks for other versions/features and raises errors that we grep for
 test_for_curl_and_openssl <- "
+#ifndef __APPLE__
 #include <ciso646>
 #ifdef _LIBCPP_VERSION
 #error Using libc++
 #endif
+#endif
 
 #include <curl/curl.h>
 #include <openssl/opensslv.h>
-#if OPENSSL_VERSION_NUMBER < 0x10002000L
-#error OpenSSL version too old
-#endif
-#if OPENSSL_VERSION_NUMBER < 0x10100000L
-#error Using OpenSSL version 1.0
-#endif
-#if OPENSSL_VERSION_NUMBER >= 0x30000000L
-#error Using OpenSSL version 3
+#if OPENSSL_VERSION_NUMBER < 0x30000000L
+#error OpenSSL version must be 3.0 or greater
 #endif
 "
 
 compile_test_program <- function(code) {
-  # Note: if we wanted to check for openssl on macOS, we'd have to set the brew
-  # path as a -I directory. But since we (currently) only run this code to
-  # determine whether we can download a Linux binary, it's not relevant.
-  runner <- "`R CMD config CXX17` `R CMD config CPPFLAGS` `R CMD config CXX17FLAGS` `R CMD config CXX17STD` -E -xc++"
+  openssl_dir <- ""
+  if (on_macos) {
+    openssl_root_dir <- get_macos_openssl_dir()
+    openssl_dir <- paste0("-I", openssl_root_dir, "/include")
+  }
+  runner <- paste(
+    R_CMD_config("CXX17"),
+    openssl_dir,
+    R_CMD_config("CPPFLAGS"),
+    R_CMD_config("CXX17FLAGS"),
+    R_CMD_config("CXX17STD"),
+    "-E",
+    "-xc++"
+  )
   suppressWarnings(system2("echo", sprintf('"%s" | %s -', code, runner), stdout = FALSE, stderr = TRUE))
 }
 
-# (built with newer devtoolset but older glibc (2.17) for broader compatibility,# like manylinux2014)
-determine_binary_from_stderr <- function(errs) {
-  if (is.null(attr(errs, "status"))) {
-    # There was no error in compiling: so we found libcurl and OpenSSL >= 1.1,
-    # openssl is < 3.0
-    cat("*** Found libcurl and OpenSSL >= 1.1\n")
-    return("linux-openssl-1.1")
-    # Else, check for dealbreakers:
-  } else if (any(grepl("Using libc++", errs, fixed = TRUE))) {
-    # Our binaries are all built with GNU stdlib so they fail with libc++
-    cat("*** Found libc++\n")
-    return(NULL)
-  } else if (header_not_found("curl/curl", errs)) {
-    cat("*** libcurl not found\n")
-    return(NULL)
-  } else if (header_not_found("openssl/opensslv", errs)) {
-    cat("*** OpenSSL not found\n")
-    return(NULL)
-  } else if (any(grepl("OpenSSL version too old", errs))) {
-    cat("*** OpenSSL found but version >= 1.0.2 is required for some features\n")
-    return(NULL)
-    # Else, determine which other binary will work
-  } else if (any(grepl("Using OpenSSL version 1.0", errs))) {
-    cat("*** Found libcurl and OpenSSL < 1.1\n")
-    return("linux-openssl-1.0")
-  } else if (any(grepl("Using OpenSSL version 3", errs))) {
-    cat("*** Found libcurl and OpenSSL >= 3.0.0\n")
-    return("linux-openssl-3.0")
+get_macos_openssl_dir <- function() {
+  openssl_root_dir <- Sys.getenv("OPENSSL_ROOT_DIR", NA)
+  header <- "openssl/opensslv.h"
+  if (is.na(openssl_root_dir) || !file.exists(file.path(openssl_root_dir, "include", header))) {
+    # try to guess default openssl include dir based on CRAN's build script
+    # https://github.com/R-macos/recipes/blob/master/build.sh#L35
+    if (identical(Sys.info()["machine"], "arm64") && file.exists(file.path("/opt/R/arm64/include", header))) {
+      openssl_root_dir <- "/opt/R/arm64"
+    } else if (identical(Sys.info()["machine"], "x86_64") && file.exists(file.path("/opt/R/x86_64/include", header))) {
+      openssl_root_dir <- "/opt/R/x86_64"
+    } else {
+      openssl_root_dir <- "/usr/local"
+    }
   }
-  NULL
+  openssl_root_dir
+}
+
+has_binary_sysreqs <- function(errs) {
+  # Check for dealbreakers:
+  if (!on_macos && any(grepl("Using libc++", errs, fixed = TRUE))) {
+    # Our linux binaries are all built with GNU stdlib so they fail with libc++
+    lg("Linux binaries incompatible with libc++")
+    return(FALSE)
+  } else if (header_not_found("curl/curl", errs)) {
+    lg("libcurl not found")
+    return(FALSE)
+  } else if (header_not_found("openssl/opensslv", errs)) {
+    lg("OpenSSL not found")
+    return(FALSE)
+  } else if (any(grepl("OpenSSL version must be 3.0 or greater", errs))) {
+    lg("OpenSSL found but version >= 3.0 is required")
+    return(FALSE)
+  } else if (is.null(attr(errs, "status"))) {
+    # Successful compile = OpenSSL >= 3.0 found
+    lg("Found libcurl and OpenSSL >= 3.0")
+    return(TRUE)
+  }
+  FALSE
 }
 
 header_not_found <- function(header, errs) {
@@ -241,6 +369,11 @@ header_not_found <- function(header, errs) {
 #### start distro ####
 
 distro <- function() {
+  # This is not part of distro but needed to enable prebuilt binaries on macos
+  if (on_macos) {
+    return(list(id = "darwin", arch = tolower(Sys.info()[["machine"]])))
+  }
+
   # The code in this script is a (potentially stale) copy of the distro package
   if (requireNamespace("distro", quietly = TRUE)) {
     # Use the version from the package, which may be updated from this
@@ -261,9 +394,7 @@ distro <- function() {
   out$id <- tolower(out$id)
   # debian unstable & testing lsb_release `version` don't include numbers but we can map from pretty name
   if (is.null(out$version) || out$version %in% c("testing", "unstable")) {
-    if (grepl("bullseye", out$codename)) {
-      out$short_version <- "11"
-    } else if (grepl("bookworm", out$codename)) {
+    if (grepl("bookworm", out$codename)) {
       out$short_version <- "12"
     }
   } else if (out$id == "ubuntu") {
@@ -351,7 +482,7 @@ find_local_source <- function() {
   )
   for (cpp_dir in cpp_dir_options) {
     if (file.exists(file.path(cpp_dir, "src/arrow/api.h"))) {
-      cat(paste0("*** Found local C++ source: '", cpp_dir, "'\n"))
+      lg("Found local C++ source: '%s'", cpp_dir)
       return(cpp_dir)
     }
   }
@@ -365,14 +496,14 @@ env_vars_as_string <- function(env_var_list) {
   stopifnot(
     length(env_var_list) == length(names(env_var_list)),
     all(grepl("^[^0-9]", names(env_var_list))),
-    all(grepl("^[A-Z0-9_]+$", names(env_var_list))),
+    all(grepl("^[a-zA-Z0-9_]+$", names(env_var_list))),
     !any(grepl("'", env_var_list, fixed = TRUE))
   )
   env_var_string <- paste0(names(env_var_list), "='", env_var_list, "'", collapse = " ")
   if (nchar(env_var_string) > 30000) {
     # This could happen if the full paths in *_SOURCE_URL were *very* long.
     # A more formal check would look at getconf ARG_MAX, but this shouldn't matter
-    cat("*** Warning: Environment variables are very long. This could cause issues on some shells.\n")
+    lg("Warning: Environment variables are very long. This could cause issues on some shells.")
   }
   env_var_string
 }
@@ -395,9 +526,15 @@ build_libarrow <- function(src_dir, dst_dir) {
   if (makeflags == "") {
     makeflags <- sprintf("-j%s", ncores)
     Sys.setenv(MAKEFLAGS = makeflags)
+  } else {
+    # Extract -j value from existing MAKEFLAGS if present
+    j_match <- regmatches(makeflags, regexpr("-j\\s*([0-9]+)", makeflags, perl = TRUE))
+    if (length(j_match) > 0) {
+      ncores <- as.integer(sub("-j\\s*", "", j_match, perl = TRUE))
+    }
   }
   if (!quietly) {
-    cat("*** Building with MAKEFLAGS=", makeflags, "\n")
+    lg("Building with MAKEFLAGS=%s", makeflags)
   }
   # Check for libarrow build dependencies:
   # * cmake
@@ -411,9 +548,9 @@ build_libarrow <- function(src_dir, dst_dir) {
     # But normally we'll just build in a tmp dir
     build_dir <- tempfile()
   }
-  options(.arrow.cleanup = c(getOption(".arrow.cleanup"), build_dir))
+  cleanup(build_dir)
 
-  env_var_list <- c(
+  env_var_list <- list(
     SOURCE_DIR = src_dir,
     BUILD_DIR = build_dir,
     DEST_DIR = dst_dir,
@@ -436,10 +573,37 @@ build_libarrow <- function(src_dir, dst_dir) {
 
   dep_source <- Sys.getenv("ARROW_DEPENDENCY_SOURCE")
   if (dep_source %in% c("", "AUTO") && !nzchar(Sys.which("pkg-config"))) {
-    cat("**** pkg-config not installed, setting ARROW_DEPENDENCY_SOURCE=BUNDLED\n")
+    lg("pkg-config not installed, setting ARROW_DEPENDENCY_SOURCE=BUNDLED", .indent = "****")
     env_var_list <- c(env_var_list, ARROW_DEPENDENCY_SOURCE = "BUNDLED")
   }
 
+  # On macOS, if not otherwise set, let's override Boost_SOURCE to be bundled
+  # Necessary due to #39590 for CRAN
+  if (on_macos) {
+    # Using lowercase (e.g. Boost_SOURCE) to match the cmake args we use already.
+    deps_to_bundle <- c("Boost", "lz4")
+    for (dep_to_bundle in deps_to_bundle) {
+      env_var <- paste0(dep_to_bundle, "_SOURCE")
+      if (Sys.getenv(env_var) == "") {
+        env_var_list <- c(env_var_list, setNames("BUNDLED", env_var))
+      }
+    }
+    # We also _do_ want to enable S3 and ZSTD by default
+    # so that binaries built on CRAN from source are fully featured
+    # but defer to the env vars if those are set
+    env_var_list <- c(
+      env_var_list,
+      ARROW_S3 = Sys.getenv("ARROW_S3", "ON"),
+      ARROW_GCS = Sys.getenv("ARROW_GCS", "ON"),
+      ARROW_WITH_ZSTD = Sys.getenv("ARROW_WITH_ZSTD", "ON")
+    )
+  }
+
+  if (on_linux_dev) {
+    # Disable mimalloc on linux devel builds, since mimalloc has spurious sanitizer failures
+    env_var_list <- c(env_var_list, ARROW_MIMALLOC = Sys.getenv("ARROW_MIMALLOC", "OFF"))
+  }
+
   env_var_list <- with_cloud_support(env_var_list)
 
   # turn_off_all_optional_features() needs to happen after
@@ -465,88 +629,130 @@ build_libarrow <- function(src_dir, dst_dir) {
   }
   env_vars <- env_vars_as_string(env_var_list)
 
-  cat("**** arrow", ifelse(quietly, "", paste("with", env_vars)), "\n")
-  status <- suppressWarnings(system(
-    paste(env_vars, "inst/build_arrow_static.sh"),
-    ignore.stdout = quietly, ignore.stderr = quietly
+  lg("arrow %s", ifelse(quietly, "", paste("with", env_vars)), .indent = "****")
+
+  build_log_path <- tempfile(fileext = ".log")
+  status <- suppressWarnings(system2(
+    "bash",
+    "inst/build_arrow_static.sh",
+    env = env_vars,
+    stdout = ifelse(quietly, build_log_path, ""),
+    stderr = ifelse(quietly, build_log_path, "")
   ))
+
   if (status != 0) {
     # It failed :(
-    cat(
-      "**** Error building Arrow C++.",
-      ifelse(env_is("ARROW_R_DEV", "true"), "", "Re-run with ARROW_R_DEV=true for debug information."),
-      "\n"
-    )
+    lg("Error building Arrow C++.", .indent = "****")
+    if (quietly) {
+      cat(
+        "**** Printing contents of build log because the build failed",
+        "while ARROW_R_DEV was set to FALSE\n"
+      )
+      cat(readLines(build_log_path), sep = "\n")
+      cat("**** Complete build log may still be present at", build_log_path, "\n")
+    }
   }
   invisible(status)
 }
 
-ensure_cmake <- function(cmake_minimum_required = "3.16") {
+ensure_cmake <- function(cmake_minimum_required = "3.26") {
   cmake <- find_cmake(version_required = cmake_minimum_required)
 
   if (is.null(cmake)) {
     # If not found, download it
-    cat("**** cmake\n")
-    CMAKE_VERSION <- Sys.getenv("CMAKE_VERSION", "3.26.4")
-    if (tolower(Sys.info()[["sysname"]]) %in% "darwin") {
+    CMAKE_VERSION <- Sys.getenv("CMAKE_VERSION", "3.31.2")
+    if (on_macos) {
       postfix <- "-macos-universal.tar.gz"
     } else if (tolower(Sys.info()[["machine"]]) %in% c("arm64", "aarch64")) {
       postfix <- "-linux-aarch64.tar.gz"
     } else if (tolower(Sys.info()[["machine"]]) == "x86_64") {
       postfix <- "-linux-x86_64.tar.gz"
     } else {
-      stop(paste0(
+      exit(paste0(
         "*** cmake was not found locally.\n",
-        "    Please make sure cmake >= ", cmake_minimum_required,
-        " is installed and available on your PATH.\n"
+        "    Please make sure cmake >= ",
+        cmake_minimum_required,
+        " is installed and available on your PATH."
       ))
     }
     cmake_binary_url <- paste0(
-      "https://github.com/Kitware/CMake/releases/download/v", CMAKE_VERSION,
-      "/cmake-", CMAKE_VERSION, postfix
+      "https://github.com/Kitware/CMake/releases/download/v",
+      CMAKE_VERSION,
+      "/cmake-",
+      CMAKE_VERSION,
+      postfix
     )
     cmake_tar <- tempfile()
     cmake_dir <- tempfile()
     download_successful <- try_download(cmake_binary_url, cmake_tar)
     if (!download_successful) {
-      cat(paste0(
+      exit(paste0(
         "*** cmake was not found locally and download failed.\n",
-        "    Make sure cmake >= ", cmake_minimum_required,
+        "    Make sure cmake >= ",
+        cmake_minimum_required,
         " is installed and available on your PATH,\n",
-        "    or download ", cmake_binary_url, "\n",
+        "    or download ",
+        cmake_binary_url,
+        "\n",
         "    and define the CMAKE environment variable.\n"
       ))
     }
     untar(cmake_tar, exdir = cmake_dir)
     unlink(cmake_tar)
-    options(.arrow.cleanup = c(getOption(".arrow.cleanup"), cmake_dir))
+    cleanup(cmake_dir)
+    # the bin dir is slightly different on macos
+    if (on_macos) {
+      bin_dir <- "CMake.app/Contents/bin"
+    } else {
+      bin_dir <- "bin"
+    }
     cmake <- paste0(
       cmake_dir,
-      "/cmake-", CMAKE_VERSION, sub(".tar.gz", "", postfix, fixed = TRUE),
-      "/bin/cmake"
+      "/cmake-",
+      CMAKE_VERSION,
+      sub(".tar.gz", "", postfix, fixed = TRUE),
+      "/",
+      bin_dir,
+      "/cmake"
     )
-  } else {
-    # Show which one we found
-    # Full source builds will always show "cmake" in the logs
-    cat(sprintf("**** cmake: %s\n", cmake))
+    lg("cmake %s", CMAKE_VERSION, .indent = "****")
   }
   cmake
 }
 
-find_cmake <- function(paths = c(
-                         Sys.getenv("CMAKE"),
-                         Sys.which("cmake"),
-                         Sys.which("cmake3")
-                       ),
-                       version_required = "3.16") {
+find_cmake <- function(
+  paths = c(
+    Sys.getenv("CMAKE"),
+    Sys.which("cmake"),
+    # CRAN has it here, not on PATH
+    if (on_macos) "/Applications/CMake.app/Contents/bin/cmake",
+    Sys.which("cmake3")
+  ),
+  version_required
+) {
   # Given a list of possible cmake paths, return the first one that exists and is new enough
   # version_required should be a string or packageVersion; numeric version
   # can be misleading (e.g. 3.10 is actually 3.1)
   for (path in paths) {
-    if (nzchar(path) && cmake_version(path) >= version_required) {
+    if (nzchar(path) && file.exists(path)) {
       # Sys.which() returns a named vector, but that plays badly with c() later
       names(path) <- NULL
-      return(path)
+      found_version <- cmake_version(path)
+      if (found_version >= version_required) {
+        # Show which one we found
+        lg("cmake %s: %s", found_version, path, .indent = "****")
+        # Stop searching here
+        return(path)
+      } else {
+        # Keep trying
+        lg("Not using cmake found at %s", path, .indent = "****")
+        if (found_version > "0") {
+          lg("Version >= %s required; found %s", version_required, found_version, .indent = "*****")
+        } else {
+          # If cmake_version() couldn't determine version, it returns 0
+          lg("Could not determine version; >= %s required", version_required, .indent = "*****")
+        }
+      }
     }
   }
   # If none found, return NULL
@@ -562,7 +768,7 @@ cmake_version <- function(cmd = "cmake") {
       package_version(sub(pat, "\\1", raw_version[which_line]))
     },
     error = function(e) {
-      return(0)
+      return("0")
     }
   )
 }
@@ -616,9 +822,7 @@ get_component_names <- function() {
   if (!deps_bash_success) {
     stop("Failed to run download_dependencies_R.sh")
   }
-  deps_df <- read.csv(csv_tempfile,
-    stringsAsFactors = FALSE, row.names = NULL, quote = "'"
-  )
+  deps_df <- read.csv(csv_tempfile, stringsAsFactors = FALSE, row.names = NULL, quote = "'")
   stopifnot(
     names(deps_df) == c("env_varname", "filename"),
     nrow(deps_df) > 0
@@ -647,8 +851,16 @@ set_thirdparty_urls <- function(env_var_list) {
   env_var_list
 }
 
-is_feature_requested <- function(env_varname, default = env_is("LIBARROW_MINIMAL", "false")) {
-  env_value <- tolower(Sys.getenv(env_varname))
+# this is generally about features that people asked for via environment variables, but
+# for some cases (like S3 when we override it in this script) we might find those in
+# env_var_list
+is_feature_requested <- function(env_varname, env_var_list, default = env_is("LIBARROW_MINIMAL", "false")) {
+  # look in the environment first, but then use the env_var_list if nothing is found
+  env_var_list_value <- env_var_list[[env_varname]]
+  if (is.null(env_var_list_value)) {
+    env_var_list_value <- ""
+  }
+  env_value <- tolower(Sys.getenv(env_varname, env_var_list_value))
   if (identical(env_value, "off")) {
     # If e.g. ARROW_MIMALLOC=OFF explicitly, override default
     requested <- FALSE
@@ -661,8 +873,9 @@ is_feature_requested <- function(env_varname, default = env_is("LIBARROW_MINIMAL
 }
 
 with_cloud_support <- function(env_var_list) {
-  arrow_s3 <- is_feature_requested("ARROW_S3")
-  arrow_gcs <- is_feature_requested("ARROW_GCS")
+  arrow_s3 <- is_feature_requested("ARROW_S3", env_var_list)
+  arrow_gcs <- is_feature_requested("ARROW_GCS", env_var_list)
+
   if (arrow_s3 || arrow_gcs) {
     # User wants S3 or GCS support.
     # Make sure that we have curl and openssl system libs
@@ -677,11 +890,6 @@ with_cloud_support <- function(env_var_list) {
       cat("**** ", start_msg, " support ", msg, "; building with ", off_flags, "\n")
     }
 
-    # Check the features
-    # This duplicates what we do with the test program above when we check
-    # capabilities for using binaries. We could consider consolidating this
-    # logic, though these use cmake in order to match exactly what we do in the
-    # libarrow build, and maybe that increases the fidelity.
     if (!cmake_find_package("CURL", NULL, env_var_list)) {
       # curl on macos should be installed, so no need to alter this for macos
       # TODO: check for apt/yum/etc. and message the right thing?
@@ -703,13 +911,15 @@ with_cloud_support <- function(env_var_list) {
 cmake_find_package <- function(pkg, version = NULL, env_var_list) {
   td <- tempfile()
   dir.create(td)
-  options(.arrow.cleanup = c(getOption(".arrow.cleanup"), td))
+  cleanup(td)
   find_package <- paste0("find_package(", pkg, " ", version, " REQUIRED)")
   writeLines(find_package, file.path(td, "CMakeLists.txt"))
   env_vars <- env_vars_as_string(env_var_list)
   cmake_cmd <- paste0(
-    "export ", env_vars,
-    " && cd ", td,
+    "export ",
+    env_vars,
+    " && cd ",
+    td,
     " && $CMAKE ",
     " -DCMAKE_EXPORT_NO_PACKAGE_REGISTRY=ON",
     " -DCMAKE_FIND_PACKAGE_NO_PACKAGE_REGISTRY=ON",
@@ -718,25 +928,141 @@ cmake_find_package <- function(pkg, version = NULL, env_var_list) {
   system(cmake_cmd, ignore.stdout = TRUE, ignore.stderr = TRUE) == 0
 }
 
-#####
+############### Main logic #############
+args <- commandArgs(TRUE)
+VERSION <- args[1]
 
-if (!test_mode && !file.exists(paste0(dst_dir, "/include/arrow/api.h"))) {
+# TESTING is set in test-nixlibs.R; it won't be set when called from configure
+test_mode <- exists("TESTING")
+
+# Prevent error with binary selection during testing.
+if (test_mode && is.na(VERSION)) {
+  VERSION <- "8.0.0.9000"
+}
+
+VERSION <- package_version(VERSION)
+dev_version <- VERSION[1, 4]
+# Small dev versions are added for R-only changes during CRAN submission
+is_release <- is.na(dev_version) || dev_version < "100"
+
+on_macos <- tolower(Sys.info()[["sysname"]]) == "darwin"
+on_windows <- tolower(Sys.info()[["sysname"]]) == "windows"
+on_linux_dev <- tolower(Sys.info()[["sysname"]]) == "linux" && grepl("devel", R.version.string)
+
+# For local debugging, set ARROW_R_DEV=TRUE to make this script print more
+quietly <- !env_is("ARROW_R_DEV", "true")
+
+# To collect dirs to rm on exit, use cleanup() to add dirs
+# we reset it to avoid errors on reruns in the same session.
+options(.arrow.cleanup = character())
+on.exit(unlink(getOption(".arrow.cleanup"), recursive = TRUE), add = TRUE)
+
+not_cran <- env_is("NOT_CRAN", "true")
+on_r_universe <- !env_is("MY_UNIVERSE", "")
+
+if (not_cran || on_r_universe) {
+  # Set more eager defaults
+  if (env_is("LIBARROW_BINARY", "")) {
+    Sys.setenv(LIBARROW_BINARY = "true")
+  }
+  if (env_is("LIBARROW_MINIMAL", "")) {
+    Sys.setenv(LIBARROW_MINIMAL = "false")
+  }
+}
+
+# The default will build from source as a fallback if a binary is not found or shouldn't be used
+# Set LIBARROW_BUILD=FALSE to ensure that we use a previously built libarrow
+# and don't fall back to a full source build
+build_ok <- !env_is("LIBARROW_BUILD", "false")
+
+# Set binary repos
+if (is_release) {
+  VERSION <- VERSION[1, 1:3]
+  VERSION_MAJOR <- unlist(VERSION)[1]
+  if (VERSION_MAJOR >= "22") {
+    VERSION_22_OR_LATER <- TRUE
+    arrow_repo <- getOption(
+      "arrow.repo",
+      sprintf("https://github.com/apache/arrow/releases/download/apache-arrow-%s/", VERSION)
+    )
+  } else {
+    VERSION_22_OR_LATER <- FALSE
+    arrow_repo <- paste0(
+      getOption("arrow.repo", sprintf("https://apache.jfrog.io/artifactory/arrow/r/%s", VERSION)),
+      "/libarrow/"
+    )
+  }
+} else {
+  VERSION_22_OR_LATER <- TRUE
+  arrow_repo <- paste0(getOption("arrow.dev_repo", "https://nightlies.apache.org/arrow/r"), "/libarrow/")
+}
+
+# Check if we're authorized to download
+download_ok <- !test_mode && !env_is("ARROW_OFFLINE_BUILD", "true")
+if (!download_ok) {
+  lg("Dependency downloading disabled. Unset ARROW_OFFLINE_BUILD to enable", .indent = "***")
+}
+# If not forbidden from downloading, check if we are offline and turn off downloading.
+# The default libarrow source build will download its source dependencies and fail
+# if they can't be retrieved.
+# But, don't do this if the user has requested a binary or a non-minimal build:
+# we should error rather than silently succeeding with a minimal build.
+if (download_ok && Sys.getenv("LIBARROW_BINARY") %in% c("false", "") && !env_is("LIBARROW_MINIMAL", "false")) {
+  if (VERSION_22_OR_LATER) {
+    download_ok <- try_download("https://github.com/apache/arrow/releases", tempfile())
+  } else {
+    download_ok <- try_download("https://apache.jfrog.io/artifactory/arrow/r/", tempfile())
+  }
+  if (!download_ok) {
+    lg("Network connection not available", .indent = "***")
+  }
+}
+
+download_libarrow_ok <- download_ok && !env_is("LIBARROW_DOWNLOAD", "false")
+
+# If we're on a dev version, look for the most recent libarrow binary version
+if (download_libarrow_ok && !is_release && !test_mode) {
+  VERSION <- find_latest_nightly(VERSION)
+}
+
+# This "tools/thirdparty_dependencies" path, within the tar file, might exist if
+# create_package_with_all_dependencies() was run, or if someone has created it
+# manually before running make build.
+# If you change this path, you also need to edit
+# `create_package_with_all_dependencies()` in install-arrow.R
+thirdparty_dependency_dir <- Sys.getenv("ARROW_THIRDPARTY_DEPENDENCY_DIR", "tools/thirdparty_dependencies")
+
+arrow_versioned <- paste0("arrow-", VERSION)
+# configure.win uses a different libarrow dir and the zip is already nested
+if (on_windows) {
+  lib_dir <- "windows"
+  dst_dir <- lib_dir
+} else {
+  lib_dir <- "libarrow"
+  dst_dir <- file.path(lib_dir, arrow_versioned)
+}
+
+api_h <- file.path(lib_dir, arrow_versioned, "include/arrow/api.h")
+
+if (!test_mode && !file.exists(api_h)) {
   # If we're working in a local checkout and have already built the libs, we
   # don't need to do anything. Otherwise,
   # (1) Look for a prebuilt binary for this version
   bin_file <- src_dir <- NULL
 
-  if (!identical(Sys.getenv("ARROW_DOWNLOADED_BINARIES"), "")) {
-    bin_zip <- Sys.getenv("ARROW_DOWNLOADED_BINARIES")
-    cat(sprintf("*** Using pre-downloaded zip for libarrow binaries: %s\n", bin_zip))
+  # Keep backwards compatibility with winlibs.R
+  bin_zip <- Sys.getenv("ARROW_DOWNLOADED_BINARIES", Sys.getenv("RWINLIB_LOCAL", NA))
+
+  if (!is.na(bin_zip)) {
+    lg("Using pre-downloaded zip for libarrow binaries: %s", bin_zip)
     if (file.exists(bin_zip)) {
       bin_file <- tempfile()
       file.copy(bin_zip, bin_file)
     } else {
-      cat(sprintf("*** File not found: %s ($ARROW_DOWNLOADED_BINARIES)\n", bin_zip))
+      lg("File not found: %s ($ARROW_DOWNLOADED_BINARIES)", bin_zip)
       bin_file <- NULL
     }
-  } else if (download_ok) {
+  } else if (download_libarrow_ok) {
     binary_flavor <- identify_binary()
     if (!is.null(binary_flavor)) {
       # The env vars say we can, and we've determined a lib that should work
@@ -749,7 +1075,7 @@ if (!test_mode && !file.exists(paste0(dst_dir, "/include/arrow/api.h"))) {
     dir.create(dst_dir, showWarnings = !quietly, recursive = TRUE)
     unzip(bin_file, exdir = dst_dir)
     unlink(bin_file)
-  } else if (build_ok) {
+  } else if (build_ok && !on_windows) {
     # (2) Find source and build it
     src_dir <- find_local_source()
     if (!is.null(src_dir)) {
@@ -760,9 +1086,9 @@ if (!test_mode && !file.exists(paste0(dst_dir, "/include/arrow/api.h"))) {
       ))
       build_libarrow(src_dir, dst_dir)
     } else {
-      cat("*** Proceeding without libarrow (no local source)\n")
+      exit("Proceeding without libarrow (no local source)")
     }
   } else {
-    cat("*** Proceeding without libarrow (build not authorized)\n")
+    exit("Proceeding without libarrow (build not authorized)")
   }
 }
