@@ -41,8 +41,25 @@ class TableTest < Test::Unit::TestCase
   end
 
   sub_test_case(".new") do
+    test("{Symbol: Arrow::Array}") do
+      schema = Arrow::Schema.new(numbers: :int64)
+      assert_equal(Arrow::Table.new(schema,
+                                    [Arrow::Int64Array.new([1, 2, 3])]),
+                   Arrow::Table.new(numbers: Arrow::Int64Array.new([1, 2, 3])))
+    end
+
+    test("{Symbol: Arrow::ChunkedArray}") do
+      chunked_array = Arrow::ChunkedArray.new([Arrow::Int64Array.new([1, 2, 3])])
+      schema = Arrow::Schema.new(numbers: :int64)
+      assert_equal(Arrow::Table.new(schema,
+                                    [Arrow::Int64Array.new([1, 2, 3])]),
+                   Arrow::Table.new(numbers: chunked_array))
+    end
+
     test("{Symbol: Arrow::Tensor}") do
-      assert_equal(Arrow::Table.new(numbers: Arrow::UInt8Array.new([1, 2, 3])),
+      schema = Arrow::Schema.new(numbers: :uint8)
+      assert_equal(Arrow::Table.new(schema,
+                                    [Arrow::UInt8Array.new([1, 2, 3])]),
                    Arrow::Table.new(numbers: Arrow::Tensor.new([1, 2, 3])))
     end
 
@@ -51,7 +68,8 @@ class TableTest < Test::Unit::TestCase
       def array_like.to_ary
         [1, 2, 3]
       end
-      assert_equal(Arrow::Table.new(numbers: Arrow::UInt8Array.new([1, 2, 3])),
+      schema = Arrow::Schema.new(numbers: :uint8)
+      assert_equal(Arrow::Table.new(schema, [Arrow::UInt8Array.new([1, 2, 3])]),
                    Arrow::Table.new(numbers: array_like))
     end
   end
@@ -69,26 +87,26 @@ class TableTest < Test::Unit::TestCase
       target_rows_raw = [nil, true, true, false, true, false, true, true]
       target_rows = Arrow::BooleanArray.new(target_rows_raw)
       assert_equal(<<-TABLE, @table.slice(target_rows).to_s)
-	 count	visible
-0	(null)	 (null)
-1	     2	false  
-2	     4	 (null)
-3	    16	true   
-4	    64	 (null)
-5	   128	 (null)
+	  count	visible
+	(uint8)	 (bool)
+0	      2	false  
+1	      4	 (null)
+2	     16	true   
+3	     64	 (null)
+4	    128	 (null)
       TABLE
     end
 
     test("Array: boolean") do
       target_rows_raw = [nil, true, true, false, true, false, true, true]
       assert_equal(<<-TABLE, @table.slice(target_rows_raw).to_s)
-	 count	visible
-0	(null)	 (null)
-1	     2	false  
-2	     4	 (null)
-3	    16	true   
-4	    64	 (null)
-5	   128	 (null)
+	  count	visible
+	(uint8)	 (bool)
+0	      2	false  
+1	      4	 (null)
+2	     16	true   
+3	     64	 (null)
+4	    128	 (null)
       TABLE
     end
 
@@ -115,89 +133,93 @@ class TableTest < Test::Unit::TestCase
 
     test("Range: positive: include end") do
       assert_equal(<<-TABLE, @table.slice(2..4).to_s)
-	count	visible
-0	    4	 (null)
-1	    8	true   
-2	   16	true   
+	  count	visible
+	(uint8)	 (bool)
+0	      4	 (null)
+1	      8	true   
+2	     16	true   
       TABLE
     end
 
     test("Range: positive: exclude end") do
       assert_equal(<<-TABLE, @table.slice(2...4).to_s)
-	count	visible
-0	    4	 (null)
-1	    8	true   
+	  count	visible
+	(uint8)	 (bool)
+0	      4	 (null)
+1	      8	true   
       TABLE
     end
 
     test("Range: negative: include end") do
       assert_equal(<<-TABLE, @table.slice(-4..-2).to_s)
-	count	visible
-0	   16	true   
-1	   32	false  
-2	   64	 (null)
+	  count	visible
+	(uint8)	 (bool)
+0	     16	true   
+1	     32	false  
+2	     64	 (null)
       TABLE
     end
 
     test("Range: negative: exclude end") do
       assert_equal(<<-TABLE, @table.slice(-4...-2).to_s)
-	count	visible
-0	   16	true   
-1	   32	false  
+	  count	visible
+	(uint8)	 (bool)
+0	     16	true   
+1	     32	false  
       TABLE
     end
 
     test("[from, to]: positive") do
       assert_equal(<<-TABLE, @table.slice(0, 2).to_s)
-	count	visible
-0	    1	true   
-1	    2	false  
+	  count	visible
+	(uint8)	 (bool)
+0	      1	true   
+1	      2	false  
       TABLE
     end
 
     test("[from, to]: negative") do
       assert_equal(<<-TABLE, @table.slice(-4, 2).to_s)
-	count	visible
-0	   16	true   
-1	   32	false  
+	  count	visible
+	(uint8)	 (bool)
+0	     16	true   
+1	     32	false  
       TABLE
     end
 
     test("{key: Number}") do
       assert_equal(<<-TABLE, @table.slice(count: 16).to_s)
-	count	visible
-0	   16	true   
+	  count	visible
+	(uint8)	 (bool)
+0	     16	true   
       TABLE
     end
 
     test("{key: String}") do
       table = Arrow::Table.new(name: Arrow::StringArray.new(["a", "b", "c"]))
       assert_equal(<<-TABLE, table.slice(name: 'b').to_s)
-	name
-0	b   
+	  name
+	(utf8)
+0	b     
       TABLE
     end
 
     test("{key: true}") do
       assert_equal(<<-TABLE, @table.slice(visible: true).to_s)
-	 count	visible
-0	     1	true   
-1	(null)	 (null)
-2	     8	true   
-3	    16	true   
-4	(null)	 (null)
-5	(null)	 (null)
+	  count	visible
+	(uint8)	 (bool)
+0	      1	true   
+1	      8	true   
+2	     16	true   
       TABLE
     end
 
     test("{key: false}") do
       assert_equal(<<-TABLE, @table.slice(visible: false).to_s)
-	 count	visible
-0	     2	false  
-1	(null)	 (null)
-2	    32	false  
-3	(null)	 (null)
-4	(null)	 (null)
+	  count	visible
+	(uint8)	 (bool)
+0	      2	false  
+1	     32	false  
       TABLE
     end
 
@@ -208,11 +230,12 @@ class TableTest < Test::Unit::TestCase
         omit("beginless range isn't supported")
       end
       assert_equal(<<-TABLE, @table.slice(count: range).to_s)
-	count	visible
-0	    1	true   
-1	    2	false  
-2	    4	 (null)
-3	    8	true   
+	  count	visible
+	(uint8)	 (bool)
+0	      1	true   
+1	      2	false  
+2	      4	 (null)
+3	      8	true   
       TABLE
     end
 
@@ -223,10 +246,11 @@ class TableTest < Test::Unit::TestCase
         omit("beginless range isn't supported")
       end
       assert_equal(<<-TABLE, @table.slice(count: range).to_s)
-	count	visible
-0	    1	true   
-1	    2	false  
-2	    4	 (null)
+	  count	visible
+	(uint8)	 (bool)
+0	      1	true   
+1	      2	false  
+2	      4	 (null)
       TABLE
     end
 
@@ -237,42 +261,43 @@ class TableTest < Test::Unit::TestCase
         omit("endless range isn't supported")
       end
       assert_equal(<<-TABLE, @table.slice(count: range).to_s)
-	count	visible
-0	   16	true   
-1	   32	false  
-2	   64	 (null)
-3	  128	 (null)
+	  count	visible
+	(uint8)	 (bool)
+0	     16	true   
+1	     32	false  
+2	     64	 (null)
+3	    128	 (null)
       TABLE
     end
 
     test("{key: Range}: include end") do
       assert_equal(<<-TABLE, @table.slice(count: 1..16).to_s)
-	count	visible
-0	    1	true   
-1	    2	false  
-2	    4	 (null)
-3	    8	true   
-4	   16	true   
+	  count	visible
+	(uint8)	 (bool)
+0	      1	true   
+1	      2	false  
+2	      4	 (null)
+3	      8	true   
+4	     16	true   
       TABLE
     end
 
     test("{key: Range}: exclude end") do
       assert_equal(<<-TABLE, @table.slice(count: 1...16).to_s)
-	count	visible
-0	    1	true   
-1	    2	false  
-2	    4	 (null)
-3	    8	true   
+	  count	visible
+	(uint8)	 (bool)
+0	      1	true   
+1	      2	false  
+2	      4	 (null)
+3	      8	true   
       TABLE
     end
 
     test("{key1: Range, key2: true}") do
       assert_equal(<<-TABLE, @table.slice(count: 0..8, visible: false).to_s)
-	 count	visible
-0	     2	false  
-1	(null)	 (null)
-2	(null)	 (null)
-3	(null)	 (null)
+	  count	visible
+	(uint8)	 (bool)
+0	      2	false  
       TABLE
     end
 
@@ -365,44 +390,47 @@ class TableTest < Test::Unit::TestCase
       test("add") do
         name_array = Arrow::StringArray.new(["a", "b", "c", "d", "e", "f", "g", "h"])
         assert_equal(<<-TABLE, @table.merge(:name => name_array).to_s)
-	count	visible	name
-0	    1	true   	a   
-1	    2	false  	b   
-2	    4	 (null)	c   
-3	    8	true   	d   
-4	   16	true   	e   
-5	   32	false  	f   
-6	   64	 (null)	g   
-7	  128	 (null)	h   
+	  count	visible	  name
+	(uint8)	 (bool)	(utf8)
+0	      1	true   	a     
+1	      2	false  	b     
+2	      4	 (null)	c     
+3	      8	true   	d     
+4	     16	true   	e     
+5	     32	false  	f     
+6	     64	 (null)	g     
+7	    128	 (null)	h     
         TABLE
       end
 
       test("remove") do
         assert_equal(<<-TABLE, @table.merge(:visible => nil).to_s)
-	count
-0	    1
-1	    2
-2	    4
-3	    8
-4	   16
-5	   32
-6	   64
-7	  128
+	  count
+	(uint8)
+0	      1
+1	      2
+2	      4
+3	      8
+4	     16
+5	     32
+6	     64
+7	    128
         TABLE
       end
 
       test("replace") do
         visible_array = Arrow::Int32Array.new([1] * @visible_array.length)
         assert_equal(<<-TABLE, @table.merge(:visible => visible_array).to_s)
-	count	visible
-0	    1	      1
-1	    2	      1
-2	    4	      1
-3	    8	      1
-4	   16	      1
-5	   32	      1
-6	   64	      1
-7	  128	      1
+	  count	visible
+	(uint8)	(int32)
+0	      1	      1
+1	      2	      1
+2	      4	      1
+3	      8	      1
+4	     16	      1
+5	     32	      1
+6	     64	      1
+7	    128	      1
         TABLE
       end
     end
@@ -412,15 +440,16 @@ class TableTest < Test::Unit::TestCase
         name_array = Arrow::StringArray.new(["a", "b", "c", "d", "e", "f", "g", "h"])
         table = Arrow::Table.new("name" => name_array)
         assert_equal(<<-TABLE, @table.merge(table).to_s)
-	count	visible	name
-0	    1	true   	a   
-1	    2	false  	b   
-2	    4	 (null)	c   
-3	    8	true   	d   
-4	   16	true   	e   
-5	   32	false  	f   
-6	   64	 (null)	g   
-7	  128	 (null)	h   
+	  count	visible	  name
+	(uint8)	 (bool)	(utf8)
+0	      1	true   	a     
+1	      2	false  	b     
+2	      4	 (null)	c     
+3	      8	true   	d     
+4	     16	true   	e     
+5	     32	false  	f     
+6	     64	 (null)	g     
+7	    128	 (null)	h     
         TABLE
       end
 
@@ -428,15 +457,16 @@ class TableTest < Test::Unit::TestCase
         visible_array = Arrow::Int32Array.new([1] * @visible_array.length)
         table = Arrow::Table.new("visible" => visible_array)
         assert_equal(<<-TABLE, @table.merge(table).to_s)
-	count	visible
-0	    1	      1
-1	    2	      1
-2	    4	      1
-3	    8	      1
-4	   16	      1
-5	   32	      1
-6	   64	      1
-7	  128	      1
+	  count	visible
+	(uint8)	(int32)
+0	      1	      1
+1	      2	      1
+2	      4	      1
+3	      8	      1
+4	     16	      1
+5	     32	      1
+6	     64	      1
+7	    128	      1
         TABLE
       end
     end
@@ -450,29 +480,31 @@ class TableTest < Test::Unit::TestCase
   sub_test_case("#remove_column") do
     test("String") do
       assert_equal(<<-TABLE, @table.remove_column("visible").to_s)
-	count
-0	    1
-1	    2
-2	    4
-3	    8
-4	   16
-5	   32
-6	   64
-7	  128
+	  count
+	(uint8)
+0	      1
+1	      2
+2	      4
+3	      8
+4	     16
+5	     32
+6	     64
+7	    128
       TABLE
     end
 
     test("Symbol") do
       assert_equal(<<-TABLE, @table.remove_column(:visible).to_s)
-	count
-0	    1
-1	    2
-2	    4
-3	    8
-4	   16
-5	   32
-6	   64
-7	  128
+	  count
+	(uint8)
+0	      1
+1	      2
+2	      4
+3	      8
+4	     16
+5	     32
+6	     64
+7	    128
       TABLE
     end
 
@@ -484,29 +516,31 @@ class TableTest < Test::Unit::TestCase
 
     test("Integer") do
       assert_equal(<<-TABLE, @table.remove_column(1).to_s)
-	count
-0	    1
-1	    2
-2	    4
-3	    8
-4	   16
-5	   32
-6	   64
-7	  128
+	  count
+	(uint8)
+0	      1
+1	      2
+2	      4
+3	      8
+4	     16
+5	     32
+6	     64
+7	    128
       TABLE
     end
 
     test("negative integer") do
       assert_equal(<<-TABLE, @table.remove_column(-1).to_s)
-	count
-0	    1
-1	    2
-2	    4
-3	    8
-4	   16
-5	   32
-6	   64
-7	  128
+	  count
+	(uint8)
+0	      1
+1	      2
+2	      4
+3	      8
+4	     16
+5	     32
+6	     64
+7	    128
       TABLE
     end
 
@@ -537,29 +571,33 @@ class TableTest < Test::Unit::TestCase
 
     test("names") do
       assert_equal(<<-TABLE, @table.select_columns(:c, :a).to_s)
-	c	a
-0	1	1
+	      c	      a
+	(uint8)	(uint8)
+0	      1	      1
       TABLE
     end
 
     test("range") do
       assert_equal(<<-TABLE, @table.select_columns(2...4).to_s)
-	c	d
-0	1	1
+	      c	      d
+	(uint8)	(uint8)
+0	      1	      1
       TABLE
     end
 
     test("indexes") do
       assert_equal(<<-TABLE, @table.select_columns(0, -1, 2).to_s)
-	a	e	c
-0	1	1	1
+	      a	      e	      c
+	(uint8)	(uint8)	(uint8)
+0	      1	      1	      1
       TABLE
     end
 
     test("mixed") do
       assert_equal(<<-TABLE, @table.select_columns(:a, -1, 2..3).to_s)
-	a	e	c	d
-0	1	1	1	1
+	      a	      e	      c	      d
+	(uint8)	(uint8)	(uint8)	(uint8)
+0	      1	      1	      1	      1
       TABLE
     end
 
@@ -568,8 +606,9 @@ class TableTest < Test::Unit::TestCase
         column.name == "a" or i.odd?
       end
       assert_equal(<<-TABLE, selected_table.to_s)
-	a	b	d
-0	1	1	1
+	      a	      b	      d
+	(uint8)	(uint8)	(uint8)
+0	      1	      1	      1
       TABLE
     end
 
@@ -578,10 +617,31 @@ class TableTest < Test::Unit::TestCase
         column.name == "a"
       end
       assert_equal(<<-TABLE, selected_table.to_s)
-	a
-0	1
+	      a
+	(uint8)
+0	      1
       TABLE
     end
+
+    test("empty result") do
+      selected_table = @table.filter([false] * @table.size).select_columns(:a)
+      assert_equal(<<-TABLE, selected_table.to_s)
+	      a
+	(uint8)
+      TABLE
+    end
+  end
+
+  sub_test_case("#column_names") do
+    test("unique") do
+      table = Arrow::Table.new(a: [1], b: [2], c: [3])
+      assert_equal(%w[a b c], table.column_names)
+    end
+
+    test("duplicated") do
+      table = Arrow::Table.new([["a", [1, 2, 3]], ["a", [4, 5, 6]]])
+      assert_equal(%w[a a], table.column_names)
+    end
   end
 
   sub_test_case("#save and .load") do
@@ -651,6 +711,31 @@ class TableTest < Test::Unit::TestCase
                                        format: :tsv,
                                        schema: @table.schema))
       end
+
+      def test_json
+        output = create_output(".json")
+        # TODO: Implement this.
+        # @table.save(output, format: :json)
+        columns = +""
+        @table.each_record.each do |record|
+          column = {
+            "count" => record.count,
+            "visible" => record.visible,
+          }
+          columns << column.to_json
+          columns << "\n"
+        end
+        if output.is_a?(String)
+          File.write(output, columns)
+        else
+          output.resize(columns.bytesize)
+          output.set_data(0, columns)
+        end
+        assert_equal(@table,
+                     Arrow::Table.load(output,
+                                       format: :json,
+                                       schema: @table.schema))
+      end
     end
 
     sub_test_case("path") do
@@ -662,6 +747,10 @@ class TableTest < Test::Unit::TestCase
           @file.path
         end
 
+        def teardown
+          GC.start # Ensure freeing Arrow::Table that refers @file.path.
+        end
+
         sub_test_case("save: auto detect") do
           test("arrow") do
             output = create_output(".arrow")
@@ -690,6 +779,11 @@ class TableTest < Test::Unit::TestCase
                                            schema: @table.schema))
           end
 
+          test("csv, return value") do
+            output = create_output(".csv")
+            assert_equal(@table, @table.save(output))
+          end
+
           test("csv.gz") do
             output = create_output(".csv.gz")
             @table.save(output)
@@ -733,10 +827,11 @@ class TableTest < Test::Unit::TestCase
             path = fixture_path("with-header.csv")
             table = Arrow::Table.load(path, skip_lines: /^\#/)
             assert_equal(<<-TABLE, table.to_s)
-	name	score
-0	alice	   10
-1	bob 	   29
-2	chris	   -1
+	  name	 score
+	(utf8)	(int8)
+0	alice 	    10
+1	bob   	    29
+2	chris 	    -1
             TABLE
           end
 
@@ -752,10 +847,11 @@ chris,-1
               CSV
             end
             assert_equal(<<-TABLE, Arrow::Table.load(file.path).to_s)
-	name	score
-0	alice	   10
-1	bob 	   29
-2	chris	   -1
+	  name	  score
+	(utf8)	(int64)
+0	alice 	     10
+1	bob   	     29
+2	chris 	     -1
             TABLE
           end
 
@@ -770,10 +866,11 @@ chris\t-1
             file.close
             table = Arrow::Table.load(file.path)
             assert_equal(<<-TABLE, table.to_s)
-	name	score
-0	alice	   10
-1	bob 	   29
-2	chris	   -1
+	  name	  score
+	(utf8)	(int64)
+0	alice 	     10
+1	bob   	     29
+2	chris 	     -1
             TABLE
           end
         end
@@ -825,26 +922,97 @@ chris\t-1
                          output.data.to_s,
                          content_type) do |port|
           input = URI("http://127.0.0.1:#{port}#{path}")
-          loaded_table = Arrow::Table.load(input)
+          loaded_table = Arrow::Table.load(input, schema: @table.schema)
           assert_equal(@table.to_s, loaded_table.to_s)
         end
       end
     end
+
+    sub_test_case("GC") do
+      def setup
+        table = Arrow::Table.new(integer: [1, 2, 3],
+                                 string: ["a", "b", "c"])
+        @buffer = Arrow::ResizableBuffer.new(1024)
+        table.save(@buffer, format: :arrow)
+        @loaded_table = Arrow::Table.load(@buffer)
+      end
+
+      def test_chunked_array
+        chunked_array = @loaded_table[0].data
+        assert_equal(@buffer,
+                     chunked_array.instance_variable_get(:@input).buffer)
+      end
+
+      def test_array
+        array = @loaded_table[0].data.chunks[0]
+        assert_equal(@buffer,
+                     array.instance_variable_get(:@input).buffer)
+      end
+
+      def test_record_batch
+        record_batch = @loaded_table.each_record_batch.first
+        assert_equal(@buffer,
+                     record_batch.instance_variable_get(:@input).buffer)
+      end
+
+      def test_record_batch_array
+        array = @loaded_table.each_record_batch.first[0].data
+        assert_equal(@buffer,
+                     array.instance_variable_get(:@input).buffer)
+      end
+
+      def test_record_batch_table
+        table = @loaded_table.each_record_batch.first.to_table
+        assert_equal(@buffer,
+                     table.instance_variable_get(:@input).buffer)
+      end
+
+      def test_slice
+        table = @loaded_table.slice(0..-1)
+        assert_equal(@buffer,
+                     table.instance_variable_get(:@input).buffer)
+      end
+
+      def test_merge
+        table = @loaded_table.merge({})
+        assert_equal(@buffer,
+                     table.instance_variable_get(:@input).buffer)
+      end
+
+      def test_remove_column
+        table = @loaded_table.remove_column(0)
+        assert_equal(@buffer,
+                     table.instance_variable_get(:@input).buffer)
+      end
+
+      def test_pack
+        table = @loaded_table.pack
+        assert_equal(@buffer,
+                     table.instance_variable_get(:@input).buffer)
+      end
+
+      def test_join
+        table = @loaded_table.join(@loaded_table, :integer)
+        assert_equal(@buffer,
+                     table.instance_variable_get(:@input).buffer)
+      end
+    end
   end
 
   test("#pack") do
     packed_table = @table.pack
     column_n_chunks = packed_table.columns.collect {|c| c.data.n_chunks}
     assert_equal([[1, 1], <<-TABLE], [column_n_chunks, packed_table.to_s])
-	count	visible
-0	    1	true   
-1	    2	false  
-2	    4	 (null)
-3	    8	true   
-4	   16	true   
-5	   32	false  
-6	   64	 (null)
-7	  128	 (null)
+	  count	visible
+	(uint8)	 (bool)
+0	      1	true   
+1	      2	false  
+2	      4	 (null)
+3	      8	true   
+4	     16	true   
+5	     32	false  
+6	     64	 (null)
+7	    128	 (null)
     TABLE
   end
 
@@ -883,6 +1051,43 @@ visible:
       test(":list") do
         assert_equal(<<-TABLE, @table.to_s(format: :list))
 ==================== 0 ====================
+count(uint8): 1
+visible(bool): true
+==================== 1 ====================
+count(uint8): 2
+visible(bool): false
+        TABLE
+      end
+
+      test(":table") do
+        assert_equal(<<-TABLE, @table.to_s(format: :table))
+	  count	visible
+	(uint8)	 (bool)
+0	      1	true   
+1	      2	false  
+        TABLE
+      end
+
+      test("invalid") do
+        message = ":format must be :column, :list, :table or nil: <:invalid>"
+        assert_raise(ArgumentError.new(message)) do
+          @table.to_s(format: :invalid)
+        end
+      end
+    end
+
+    sub_test_case(":show_column_type") do
+      def setup
+        columns = {
+          "count" => Arrow::UInt8Array.new([1, 2]),
+          "visible" => Arrow::BooleanArray.new([true, false]),
+        }
+        @table = Arrow::Table.new(columns)
+      end
+
+      test(":list") do
+        assert_equal(<<-TABLE, @table.to_s(format: :list, show_column_type: false))
+==================== 0 ====================
 count: 1
 visible: true
 ==================== 1 ====================
@@ -892,19 +1097,12 @@ visible: false
       end
 
       test(":table") do
-        assert_equal(<<-TABLE, @table.to_s(format: :table))
+        assert_equal(<<-TABLE, @table.to_s(format: :table, show_column_type: false))
 	count	visible
 0	    1	true   
 1	    2	false  
         TABLE
       end
-
-      test("invalid") do
-        message = ":format must be :column, :list, :table or nil: <:invalid>"
-        assert_raise(ArgumentError.new(message)) do
-          @table.to_s(format: :invalid)
-        end
-      end
     end
 
     sub_test_case("#==") do
@@ -932,13 +1130,14 @@ visible: false
     test("Array: boolean") do
       filter = [nil, true, true, false, true, false, true, true]
       assert_equal(<<-TABLE, @table.filter(filter, @options).to_s)
-	 count	visible
-0	(null)	 (null)
-1	     2	false  
-2	     4	 (null)
-3	    16	true   
-4	    64	 (null)
-5	   128	 (null)
+	  count	visible
+	(uint8)	 (bool)
+0	 (null)	 (null)
+1	      2	false  
+2	      4	 (null)
+3	     16	true   
+4	     64	 (null)
+5	    128	 (null)
       TABLE
     end
 
@@ -946,13 +1145,14 @@ visible: false
       array = [nil, true, true, false, true, false, true, true]
       filter = Arrow::BooleanArray.new(array)
       assert_equal(<<-TABLE, @table.filter(filter, @options).to_s)
-	 count	visible
-0	(null)	 (null)
-1	     2	false  
-2	     4	 (null)
-3	    16	true   
-4	    64	 (null)
-5	   128	 (null)
+	  count	visible
+	(uint8)	 (bool)
+0	 (null)	 (null)
+1	      2	false  
+2	      4	 (null)
+3	     16	true   
+4	     64	 (null)
+5	    128	 (null)
       TABLE
     end
 
@@ -964,13 +1164,14 @@ visible: false
       ]
       filter = Arrow::ChunkedArray.new(filter_chunks)
       assert_equal(<<-TABLE, @table.filter(filter, @options).to_s)
-	 count	visible
-0	(null)	 (null)
-1	     2	false  
-2	     4	 (null)
-3	    16	true   
-4	    64	 (null)
-5	   128	 (null)
+	  count	visible
+	(uint8)	 (bool)
+0	 (null)	 (null)
+1	      2	false  
+2	      4	 (null)
+3	     16	true   
+4	     64	 (null)
+5	    128	 (null)
       TABLE
     end
   end
@@ -979,20 +1180,22 @@ visible: false
     test("Arrow: boolean") do
       indices = [1, 0, 2]
       assert_equal(<<-TABLE, @table.take(indices).to_s)
-	count	visible
-0	    2	false  
-1	    1	true   
-2	    4	 (null)
+	  count	visible
+	(uint8)	 (bool)
+0	      2	false  
+1	      1	true   
+2	      4	 (null)
       TABLE
     end
 
     test("Arrow::Array") do
       indices = Arrow::Int16Array.new([1, 0, 2])
       assert_equal(<<-TABLE, @table.take(indices).to_s)
-	count	visible
-0	    2	false  
-1	    1	true   
-2	    4	 (null)
+	  count	visible
+	(uint8)	 (bool)
+0	      2	false  
+1	      1	true   
+2	      4	 (null)
       TABLE
     end
 
@@ -1003,10 +1206,11 @@ visible: false
       ]
       indices = Arrow::ChunkedArray.new(chunks)
       assert_equal(<<-TABLE, @table.take(indices).to_s)
-	count	visible
-0	    2	false  
-1	    1	true   
-2	    4	 (null)
+	  count	visible
+	(uint8)	 (bool)
+0	      2	false  
+1	      1	true   
+2	      4	 (null)
       TABLE
     end
   end
@@ -1018,14 +1222,28 @@ visible: false
       table2 = Arrow::Table.new(b: [false])
       concatenated = table1.concatenate([table2], unify_schemas: true)
       assert_equal(<<-TABLE, concatenated.to_s)
-	a	b
-0	true	false
-1	(null)	false
+	     a	     b
+	(bool)	(bool)
+0	true  	false 
+1	(null)	false 
       TABLE
     end
   end
 
   sub_test_case("#join") do
+    test("keys: nil (natural join)") do
+      table1 = Arrow::Table.new(key: [1, 2, 3],
+                                number: [10, 20, 30])
+      table2 = Arrow::Table.new(key: [3, 1],
+                                string: ["three", "one"])
+      assert_equal(Arrow::Table.new([
+                                      ["key", [1, 3]],
+                                      ["number", [10, 30]],
+                                      ["string", ["one", "three"]],
+                                    ]),
+                   table1.join(table2))
+    end
+
     test("keys: String") do
       table1 = Arrow::Table.new(key: [1, 2, 3],
                                 number: [10, 20, 30])
@@ -1034,7 +1252,6 @@ visible: false
       assert_equal(Arrow::Table.new([
                                       ["key", [1, 3]],
                                       ["number", [10, 30]],
-                                      ["key", [1, 3]],
                                       ["string", ["one", "three"]],
                                     ]),
                    table1.join(table2, "key"))
@@ -1048,12 +1265,25 @@ visible: false
       assert_equal(Arrow::Table.new([
                                       ["key", [1, 3]],
                                       ["number", [10, 30]],
-                                      ["key", [1, 3]],
                                       ["string", ["one", "three"]],
                                     ]),
                    table1.join(table2, :key))
     end
 
+    test("keys: [String]") do
+      table1 = Arrow::Table.new(key: [1, 2, 3],
+                                number: [10, 20, 30])
+      table2 = Arrow::Table.new(key: [3, 1],
+                                string: ["three", "one"])
+      assert_equal(Arrow::Table.new([
+                                      ["key", [1, 3]],
+                                      ["number", [10, 30]],
+                                      ["key", [1, 3]],
+                                      ["string", ["one", "three"]],
+                                    ]),
+                   table1.join(table2, ["key"]))
+    end
+
     test("keys: [String, Symbol]") do
       table1 = Arrow::Table.new(key1: [1, 1, 2, 2],
                                 key2: [10, 100, 20, 200],
@@ -1083,7 +1313,9 @@ visible: false
                                       ["right_key", [1, 3]],
                                       ["string", ["one", "three"]],
                                     ]),
-                   table1.join(table2, {left: "left_key", right: :right_key}))
+                   table1.join(table2,
+                               {left: "left_key", right: :right_key},
+                               type: :inner))
     end
 
     test("keys: {left: [String, Symbol], right: [Symbol, String]}") do
@@ -1105,10 +1337,11 @@ visible: false
                                {
                                  left: ["left_key1", :left_key2],
                                  right: [:right_key1, "right_key2"],
-                               }))
+                               },
+                               type: :inner))
     end
 
-    test("type:") do
+    test("type: :left_outer") do
       table1 = Arrow::Table.new(key: [1, 2, 3],
                                 number: [10, 20, 30])
       table2 = Arrow::Table.new(key: [3, 1],
@@ -1116,12 +1349,85 @@ visible: false
       assert_equal(Arrow::Table.new([
                                       ["key", [1, 3, 2]],
                                       ["number", [10, 30, 20]],
-                                      ["key", [1, 3, nil]],
                                       ["string", ["one", "three", nil]],
                                     ]),
                    table1.join(table2, "key", type: :left_outer))
     end
 
+    test("type: :right_outer") do
+      table1 = Arrow::Table.new(key: [1, 2, 3],
+                                number: [10, 20, 30])
+      table2 = Arrow::Table.new(key: [3, 1],
+                                string: ["three", "one"])
+      assert_equal(Arrow::Table.new([
+                                      ["key", [1, 3]],
+                                      ["number", [10, 30]],
+                                      ["string", ["one", "three"]],
+                                    ]),
+                   table1.join(table2, "key", type: :right_outer))
+    end
+
+    test("type: :full_outer") do
+      table1 = Arrow::Table.new(key: [1, 2, 3],
+                                number: [10, 20, 30])
+      table2 = Arrow::Table.new(key: [3, 1],
+                                string: ["three", "one"])
+      assert_equal(Arrow::Table.new([
+                                      ["key", [1, 3, 2]],
+                                      ["number", [10, 30, 20]],
+                                      ["string", ["one", "three", nil]],
+                                    ]),
+                   table1.join(table2, "key", type: :full_outer))
+    end
+
+    test("type: :left_semi") do
+      table1 = Arrow::Table.new(key: [1, 2, 3],
+                                number: [10, 20, 30])
+      table2 = Arrow::Table.new(key: [3, 1],
+                                string: ["three", "one"])
+      assert_equal(Arrow::Table.new([
+                                      ["key", [1, 3]],
+                                      ["number", [10, 30]],
+                                    ]),
+                   table1.join(table2, "key", type: :left_semi))
+    end
+
+    test("type: :right_semi") do
+      table1 = Arrow::Table.new(key: [1, 2, 3],
+                                number: [10, 20, 30])
+      table2 = Arrow::Table.new(key: [3, 1],
+                                string: ["three", "one"])
+      assert_equal(Arrow::Table.new([
+                                      ["key", [3, 1]],
+                                      ["string", ["three", "one"]],
+                                    ]),
+                   table1.join(table2, "key", type: :right_semi))
+    end
+
+    test("type: :left_anti") do
+      table1 = Arrow::Table.new(key: [1, 2, 3],
+                                number: [10, 20, 30])
+      table2 = Arrow::Table.new(key: [3, 1],
+                                string: ["three", "one"])
+      assert_equal(Arrow::Table.new([
+                                      ["key", [2]],
+                                      ["number", [20]],
+                                    ]),
+                   table1.join(table2, "key", type: :left_anti))
+    end
+
+    test("type: :right_anti") do
+      table1 = Arrow::Table.new(key: [1, 2, 3],
+                                number: [10, 20, 30])
+      table2 = Arrow::Table.new(key: [3, 1],
+                                string: ["three", "one"])
+      assert_equal(Arrow::Table.new([
+                                      ["key", Arrow::ChunkedArray.new(:uint8)],
+                                      ["string", Arrow::ChunkedArray.new(:string)],
+                                    ]),
+                   table1.join(table2, "key", type: :right_anti))
+    end
+
     test("left_outputs: & right_outputs:") do
       table1 = Arrow::Table.new(key: [1, 2, 3],
                                 number: [10, 20, 30])
@@ -1135,5 +1441,94 @@ visible: false
                                left_outputs: ["key", "number"],
                                right_outputs: ["string"]))
     end
+
+    test("left_outputs: & type: :inner") do
+      table1 = Arrow::Table.new(key: [1, 2, 3],
+                                number: [10, 20, 30])
+      table2 = Arrow::Table.new(key: [3, 1],
+                                string: ["three", "one"])
+      assert_equal(Arrow::Table.new([
+                                      ["key", [1, 3]],
+                                      ["number", [10, 30]],
+                                      ["key", [1, 3]],
+                                      ["string", ["one", "three"]]
+                                    ]),
+                   table1.join(table2,
+                               type: :inner,
+                               left_outputs: table1.column_names,
+                               right_outputs: table2.column_names))
+    end
+
+    test("left_outputs: & type: :left_outer") do
+      table1 = Arrow::Table.new(key: [1, 2, 3],
+                                number: [10, 20, 30])
+      table2 = Arrow::Table.new(key: [3, 1],
+                                string: ["three", "one"])
+      assert_equal(Arrow::Table.new([
+                                      ["key", [1, 3, 2]],
+                                      ["number", [10, 30, 20]],
+                                      ["key", [1, 3, nil]],
+                                      ["string", ["one", "three", nil]],
+                                    ]),
+                   table1.join(table2,
+                               type: :left_outer,
+                               left_outputs: table1.column_names,
+                               right_outputs: table2.column_names))
+    end
+
+    test("left_outputs: & type: :right_outer") do
+      table1 = Arrow::Table.new(key: [1, 2, 3],
+                                number: [10, 20, 30])
+      table2 = Arrow::Table.new(key: [3, 1],
+                                string: ["three", "one"])
+      assert_equal(Arrow::Table.new([
+                                      ["key", [1, 3]],
+                                      ["number", [10, 30]],
+                                      ["key", [1, 3]],
+                                      ["string", ["one", "three"]],
+                                    ]),
+                   table1.join(table2,
+                               type: :right_outer,
+                               left_outputs: table1.column_names,
+                               right_outputs: table2.column_names))
+    end
+
+    test("left_outputs: & type: :full_outer") do
+      table1 = Arrow::Table.new(key: [1, 2, 3],
+                                number: [10, 20, 30])
+      table2 = Arrow::Table.new(key: [3, 1],
+                                string: ["three", "one"])
+      assert_equal(Arrow::Table.new([
+                                      ["key", [1, 3, 2]],
+                                      ["number", [10, 30, 20]],
+                                      ["key", [1, 3, nil]],
+                                      ["string", ["one", "three", nil]],
+                                    ]),
+                   table1.join(table2,
+                               type: :full_outer,
+                               left_outputs: table1.column_names,
+                               right_outputs: table2.column_names))
+    end
+
+    test("left_suffix: & keys: [String]") do
+      table1 = Arrow::Table.new(key1: [1, 1, 2, 2],
+                                key2: [10, 100, 20, 200],
+                                number: [1010, 1100, 2020, 2200])
+      table2 = Arrow::Table.new(key1: [1, 2, 2],
+                                key2: [100, 20, 50],
+                                string: ["1-100", "2-20", "2-50"])
+      assert_equal(Arrow::Table.new([
+                                      ["key1_left", [1, 2]],
+                                      ["key2_left", [100, 20]],
+                                      ["number", [1100, 2020]],
+                                      ["key1_right", [1, 2]],
+                                      ["key2_right", [100, 20]],
+                                      ["string", ["1-100", "2-20"]],
+                                    ]),
+                   table1.join(table2,
+                               ["key1", "key2"],
+                               left_suffix: "_left",
+                               right_suffix: "_right"))
+    end
   end
 end
