@@ -17,15 +17,22 @@
 
 class TestBooleanStatistics < Test::Unit::TestCase
   def setup
-    file = Tempfile.open(["data", ".parquet"])
-    array = Arrow::BooleanArray.new([nil, false, true])
-    table = Arrow::Table.new("boolean" => array)
-    writer = Parquet::ArrowFileWriter.new(table.schema, file.path)
-    chunk_size = 1024
-    writer.write_table(table, chunk_size)
-    writer.close
-    reader = Parquet::ArrowFileReader.new(file.path)
-    @statistics = reader.metadata.get_row_group(0).get_column_chunk(0).statistics
+    Tempfile.create(["data", ".parquet"]) do |file|
+      array = Arrow::BooleanArray.new([nil, false, true])
+      table = Arrow::Table.new("boolean" => array)
+      Parquet::ArrowFileWriter.open(table.schema, file.path) do |writer|
+        chunk_size = 1024
+        writer.write_table(table, chunk_size)
+        writer.close
+      end
+      Parquet::ArrowFileReader.open(file.path) do |reader|
+        @statistics =
+          reader.metadata.get_row_group(0).get_column_chunk(0).statistics
+        yield
+        @statistics = nil
+      end
+      GC.start # Ensure freeing @statistics that refers file.path.
+    end
   end
 
   def test_min
